#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

PROCESSOR_FEATURE_MAX = 64
MAX_WOW64_SHARED_ENTRIES = 16

# IRP Major Function Codes
IRP_MJ_CREATE = 0x00
IRP_MJ_CREATE_NAMED_PIPE = 0x01
IRP_MJ_CLOSE = 0x02
IRP_MJ_READ = 0x03
IRP_MJ_WRITE = 0x04
IRP_MJ_QUERY_INFORMATION = 0x05
IRP_MJ_SET_INFORMATION = 0x06
IRP_MJ_QUERY_EA = 0x07
IRP_MJ_SET_EA = 0x08
IRP_MJ_FLUSH_BUFFERS = 0x09
IRP_MJ_QUERY_VOLUME_INFORMATION = 0x0a
IRP_MJ_SET_VOLUME_INFORMATION = 0x0b
IRP_MJ_DIRECTORY_CONTROL = 0x0c
IRP_MJ_FILE_SYSTEM_CONTROL = 0x0d
IRP_MJ_DEVICE_CONTROL = 0x0e
IRP_MJ_INTERNAL_DEVICE_CONTROL = 0x0f
IRP_MJ_SHUTDOWN = 0x10
IRP_MJ_LOCK_CONTROL = 0x11
IRP_MJ_CLEANUP = 0x12
IRP_MJ_CREATE_MAILSLOT = 0x13
IRP_MJ_QUERY_SECURITY = 0x14
IRP_MJ_SET_SECURITY = 0x15
IRP_MJ_POWER = 0x16
IRP_MJ_SYSTEM_CONTROL = 0x17
IRP_MJ_DEVICE_CHANGE = 0x18
IRP_MJ_QUERY_QUOTA = 0x19
IRP_MJ_SET_QUOTA = 0x1a
IRP_MJ_PNP = 0x1b
IRP_MJ_MAXIMUM_FUNCTION = 0x1b

# Device Types
device_name_unknown =              0x00000000
FILE_DEVICE_BEEP =                 0x00000001
FILE_DEVICE_CD_ROM =               0x00000002
FILE_DEVICE_CD_ROM_FILE_SYSTEM =   0x00000003
FILE_DEVICE_CONTROLLER =           0x00000004
FILE_DEVICE_DATALINK =             0x00000005
FILE_DEVICE_DFS =                  0x00000006
FILE_DEVICE_DISK =                 0x00000007
FILE_DEVICE_DISK_FILE_SYSTEM =     0x00000008
FILE_DEVICE_FILE_SYSTEM =          0x00000009
FILE_DEVICE_INPORT_PORT =          0x0000000a
FILE_DEVICE_KEYBOARD =             0x0000000b
FILE_DEVICE_MAILSLOT =             0x0000000c
FILE_DEVICE_MIDI_IN =              0x0000000d
FILE_DEVICE_MIDI_OUT =             0x0000000e
FILE_DEVICE_MOUSE =                0x0000000f
FILE_DEVICE_MULTI_UNC_PROVIDER =   0x00000010
FILE_DEVICE_NAMED_PIPE =           0x00000011
FILE_DEVICE_NETWORK =              0x00000012
FILE_DEVICE_NETWORK_BROWSER =      0x00000013
FILE_DEVICE_NETWORK_FILE_SYSTEM =  0x00000014
FILE_DEVICE_NULL =                 0x00000015
FILE_DEVICE_PARALLEL_PORT =        0x00000016
FILE_DEVICE_PHYSICAL_NETCARD =     0x00000017
FILE_DEVICE_PRINTER =              0x00000018
FILE_DEVICE_SCANNER =              0x00000019
FILE_DEVICE_SERIAL_MOUSE_PORT =    0x0000001a
FILE_DEVICE_SERIAL_PORT =          0x0000001b
FILE_DEVICE_SCREEN =               0x0000001c
FILE_DEVICE_SOUND =                0x0000001d
FILE_DEVICE_STREAMS =              0x0000001e
FILE_DEVICE_TAPE =                 0x0000001f
FILE_DEVICE_TAPE_FILE_SYSTEM =     0x00000020
FILE_DEVICE_TRANSPORT =            0x00000021
FILE_DEVICE_UNKNOWN =              0x00000022
FILE_DEVICE_VIDEO =                0x00000023
FILE_DEVICE_VIRTUAL_DISK =         0x00000024
FILE_DEVICE_WAVE_IN =              0x00000025
FILE_DEVICE_WAVE_OUT =             0x00000026
FILE_DEVICE_8042_PORT =            0x00000027
FILE_DEVICE_NETWORK_REDIRECTOR =   0x00000028
FILE_DEVICE_BATTERY =              0x00000029
FILE_DEVICE_BUS_EXTENDER =         0x0000002a
FILE_DEVICE_MODEM =                0x0000002b
FILE_DEVICE_VDM =                  0x0000002c
FILE_DEVICE_MASS_STORAGE =         0x0000002d
FILE_DEVICE_SMB =                  0x0000002e
FILE_DEVICE_KS =                   0x0000002f
FILE_DEVICE_CHANGER =              0x00000030
FILE_DEVICE_SMARTCARD =            0x00000031
FILE_DEVICE_ACPI =                 0x00000032
FILE_DEVICE_DVD =                  0x00000033
FILE_DEVICE_FULLSCREEN_VIDEO =     0x00000034
FILE_DEVICE_DFS_FILE_SYSTEM =      0x00000035
FILE_DEVICE_DFS_VOLUME =           0x00000036
FILE_DEVICE_SERENUM =              0x00000037
FILE_DEVICE_TERMSRV =              0x00000038
FILE_DEVICE_KSEC =                 0x00000039
FILE_DEVICE_FIPS =                 0x0000003A
FILE_DEVICE_INFINIBAND =           0x0000003B
FILE_DEVICE_VMBUS =                0x0000003E
FILE_DEVICE_CRYPT_PROVIDER =       0x0000003F
FILE_DEVICE_WPD =                  0x00000040
FILE_DEVICE_BLUETOOTH =            0x00000041
FILE_DEVICE_MT_COMPOSITE =         0x00000042
FILE_DEVICE_MT_TRANSPORT =         0x00000043
FILE_DEVICE_BIOMETRIC =            0x00000044
FILE_DEVICE_PMI =                  0x00000045

DEVICE_TYPE = {
    device_name_unknown: "device_name_unknown",
    FILE_DEVICE_BEEP: "FILE_DEVICE_BEEP",
    FILE_DEVICE_CD_ROM: "FILE_DEVICE_CD_ROM",
    FILE_DEVICE_CD_ROM_FILE_SYSTEM: "FILE_DEVICE_CD_ROM_FILE_SYSTEM",
    FILE_DEVICE_CONTROLLER: "FILE_DEVICE_CONTROLLER",
    FILE_DEVICE_DATALINK: "FILE_DEVICE_DATALINK",
    FILE_DEVICE_DFS: "FILE_DEVICE_DFS",
    FILE_DEVICE_DISK: "FILE_DEVICE_DISK",
    FILE_DEVICE_DISK_FILE_SYSTEM: "FILE_DEVICE_DISK_FILE_SYSTEM",
    FILE_DEVICE_FILE_SYSTEM: "FILE_DEVICE_FILE_SYSTEM",
    FILE_DEVICE_INPORT_PORT: "FILE_DEVICE_INPORT_PORT",
    FILE_DEVICE_KEYBOARD: "FILE_DEVICE_KEYBOARD",
    FILE_DEVICE_MAILSLOT: "FILE_DEVICE_MAILSLOT",
    FILE_DEVICE_MIDI_IN: "FILE_DEVICE_MIDI_IN",
    FILE_DEVICE_MIDI_OUT: "FILE_DEVICE_MIDI_OUT",
    FILE_DEVICE_MOUSE: "FILE_DEVICE_MOUSE",
    FILE_DEVICE_MULTI_UNC_PROVIDER: "FILE_DEVICE_MULTI_UNC_PROVIDER",
    FILE_DEVICE_NAMED_PIPE: "FILE_DEVICE_NAMED_PIPE",
    FILE_DEVICE_NETWORK: "FILE_DEVICE_NETWORK",
    FILE_DEVICE_NETWORK_BROWSER: "FILE_DEVICE_NETWORK_BROWSER",
    FILE_DEVICE_NETWORK_FILE_SYSTEM: "FILE_DEVICE_NETWORK_FILE_SYSTEM",
    FILE_DEVICE_NULL: "FILE_DEVICE_NULL",
    FILE_DEVICE_PARALLEL_PORT: "FILE_DEVICE_PARALLEL_PORT",
    FILE_DEVICE_PHYSICAL_NETCARD: "FILE_DEVICE_PHYSICAL_NETCARD",
    FILE_DEVICE_PRINTER: "FILE_DEVICE_PRINTER",
    FILE_DEVICE_SCANNER: "FILE_DEVICE_SCANNER",
    FILE_DEVICE_SERIAL_MOUSE_PORT: "FILE_DEVICE_SERIAL_MOUSE_PORT",
    FILE_DEVICE_SERIAL_PORT: "FILE_DEVICE_SERIAL_PORT",
    FILE_DEVICE_SCREEN: "FILE_DEVICE_SCREEN",
    FILE_DEVICE_SOUND: "FILE_DEVICE_SOUND",
    FILE_DEVICE_STREAMS: "FILE_DEVICE_STREAMS",
    FILE_DEVICE_TAPE: "FILE_DEVICE_TAPE",
    FILE_DEVICE_TAPE_FILE_SYSTEM: "FILE_DEVICE_TAPE_FILE_SYSTEM",
    FILE_DEVICE_TRANSPORT: "FILE_DEVICE_TRANSPORT",
    FILE_DEVICE_UNKNOWN: "FILE_DEVICE_UNKNOWN",
    FILE_DEVICE_VIDEO: "FILE_DEVICE_VIDEO",
    FILE_DEVICE_VIRTUAL_DISK: "FILE_DEVICE_VIRTUAL_DISK",
    FILE_DEVICE_WAVE_IN: "FILE_DEVICE_WAVE_IN",
    FILE_DEVICE_WAVE_OUT: "FILE_DEVICE_WAVE_OUT",
    FILE_DEVICE_8042_PORT: "FILE_DEVICE_8042_PORT",
    FILE_DEVICE_NETWORK_REDIRECTOR: "FILE_DEVICE_NETWORK_REDIRECTOR",
    FILE_DEVICE_BATTERY: "FILE_DEVICE_BATTERY",
    FILE_DEVICE_BUS_EXTENDER: "FILE_DEVICE_BUS_EXTENDER",
    FILE_DEVICE_MODEM: "FILE_DEVICE_MODEM",
    FILE_DEVICE_VDM: "FILE_DEVICE_VDM",
    FILE_DEVICE_MASS_STORAGE: "FILE_DEVICE_MASS_STORAGE",
    FILE_DEVICE_SMB: "FILE_DEVICE_SMB",
    FILE_DEVICE_KS: "FILE_DEVICE_KS",
    FILE_DEVICE_CHANGER: "FILE_DEVICE_CHANGER",
    FILE_DEVICE_SMARTCARD: "FILE_DEVICE_SMARTCARD",
    FILE_DEVICE_ACPI: "FILE_DEVICE_ACPI",
    FILE_DEVICE_DVD: "FILE_DEVICE_DVD",
    FILE_DEVICE_FULLSCREEN_VIDEO: "FILE_DEVICE_FULLSCREEN_VIDEO",
    FILE_DEVICE_DFS_FILE_SYSTEM: "FILE_DEVICE_DFS_FILE_SYSTEM",
    FILE_DEVICE_DFS_VOLUME: "FILE_DEVICE_DFS_VOLUME",
    FILE_DEVICE_SERENUM: "FILE_DEVICE_SERENUM",
    FILE_DEVICE_TERMSRV: "FILE_DEVICE_TERMSRV",
    FILE_DEVICE_KSEC: "FILE_DEVICE_KSEC",
    FILE_DEVICE_FIPS: "FILE_DEVICE_FIPS",
    FILE_DEVICE_INFINIBAND: "FILE_DEVICE_INFINIBAND",
    FILE_DEVICE_VMBUS: "FILE_DEVICE_VMBUS",
    FILE_DEVICE_CRYPT_PROVIDER: "FILE_DEVICE_CRYPT_PROVIDER",
    FILE_DEVICE_WPD: "FILE_DEVICE_WPD",
    FILE_DEVICE_BLUETOOTH: "FILE_DEVICE_BLUETOOTH",
    FILE_DEVICE_MT_COMPOSITE: "FILE_DEVICE_MT_COMPOSITE",
    FILE_DEVICE_MT_TRANSPORT: "FILE_DEVICE_MT_TRANSPORT",
    FILE_DEVICE_BIOMETRIC: "FILE_DEVICE_BIOMETRIC",
    FILE_DEVICE_PMI: "FILE_DEVICE_PMI"
}

# file access
FILE_ANY_ACCESS = 0
FILE_READ_DATA = 1
FILE_WRITE_DATA = 2

DEVICE_ACCESS = {
    FILE_ANY_ACCESS: "FILE_ANY_ACCESS",
    FILE_READ_DATA: "FILE_READ_DATA",
    FILE_WRITE_DATA: "FILE_WRITE_DATA",
    FILE_READ_DATA | FILE_WRITE_DATA: "FILE_READ_DATA | FILE_WRITE_DATA"
}

# Transfer Type
METHOD_BUFFERED = 0
METHOD_IN_DIRECT = 1
METHOD_OUT_DIRECT = 2
METHOD_NEITHER = 3

DEVICE_METHOD = {
    METHOD_BUFFERED: "METHOD_BUFFERED",
    METHOD_IN_DIRECT: "METHOD_IN_DIRECT",
    METHOD_OUT_DIRECT: "METHOD_OUT_DIRECT",
    METHOD_NEITHER: "METHOD_NEITHER",
}


DO_BUFFERED_IO                 = 0x00000004
DO_EXCLUSIVE                   = 0x00000008
DO_DIRECT_IO                   = 0x00000010
DO_MAP_IO_BUFFER               = 0x00000020
DO_DEVICE_INITIALIZING         = 0x00000080
DO_SHUTDOWN_REGISTERED         = 0x00000800
DO_BUS_ENUMERATED_DEVICE       = 0x00001000
DO_POWER_PAGABLE               = 0x00002000
DO_POWER_INRUSH                = 0x00004000



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.arch.x86 import *
from qiling.const import *
from qiling.os.const import *
from qiling.os.posix.posix import QlOsPosix
from .const import *

class QlOsFreebsd(QlOsPosix):
    def __init__(self, ql):
        super(QlOsFreebsd, self).__init__(ql)
        self.pid = self.profile.getint("KERNEL","pid")
        self.load()


    def load(self):   
        self.ql.hook_insn(self.hook_syscall, UC_X86_INS_SYSCALL)
        self.gdtm = GDTManager(self.ql)
        ql_x86_register_cs(self)
        ql_x86_register_ds_ss_es(self)


    def hook_syscall(self, intno= None):
        return self.load_syscall()


    def run(self):
        if self.ql.exit_point is not None:
            self.exit_point = self.ql.exit_point

        if  self.ql.entry_point is not None:
            self.ql.loader.elf_entry = self.ql.entry_point            
        
        try:
            if self.ql.code:
                self.ql.emu_start(self.entry_point, (self.entry_point + len(self.ql.code)), self.ql.timeout, self.ql.count)
            else:
                if self.ql.loader.elf_entry != self.ql.loader.entry_point:
                    self.ql.emu_start(self.ql.loader.entry_point, self.ql.loader.elf_entry, self.ql.timeout)
                    self.ql.enable_lib_patch()
                                        
                self.ql.emu_start(self.ql.loader.elf_entry, self.exit_point, self.ql.timeout, self.ql.count)
                
        except UcError:
            self.emu_error()
            raise
    



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#


from qiling.const import *
from qiling.os.linux.thread import *
from qiling.const import *
from qiling.os.posix.filestruct import *
from qiling.os.filestruct import *
from qiling.os.posix.const_mapping import *
from qiling.exception import *

def ql_syscall_rt_sigaction(ql, rt_sigaction_signum, rt_sigaction_act, rt_sigaction_oldact, *args, **kw):
    if rt_sigaction_oldact != 0:
        if ql.os.sigaction_act[rt_sigaction_signum] == 0:
            ql.mem.write(rt_sigaction_oldact, b'\x00' * 20)
        else:
            data = b''
            for key in ql.os.sigaction_act[rt_sigaction_signum]:
                data += ql.pack32(key)
            ql.mem.write(rt_sigaction_oldact, data)

    if rt_sigaction_act != 0:
        data = []
        for key in range(5):
            data.append(ql.unpack32(ql.mem.read(rt_sigaction_act + 4 * key, 4)))
        ql.os.sigaction_act[rt_sigaction_signum] = data

    regreturn = 0
    return regreturn


def ql_syscall_rt_sigprocmask(ql, rt_sigprocmask_how, rt_sigprocmask_nset, rt_sigprocmask_oset, rt_sigprocmask_sigsetsize, *args, **kw):
    SIG_BLOCK = 0x0
    SIG_UNBLOCK = 0x1

    if rt_sigprocmask_how == SIG_BLOCK:
        pass

    regreturn = 0
    return regreturn


def ql_syscall_signal(ql, sig, __sighandler_t, *args, **kw):
    regreturn = 0
    return regreturn



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import stat, itertools, pathlib

from multiprocessing import Process


from qiling.const import *
from qiling.os.linux.thread import *
from qiling.const import *
from qiling.os.posix.filestruct import *
from qiling.os.filestruct import *
from qiling.os.posix.const_mapping import *
from qiling.exception import *
from qiling.os.stat import *

def ql_syscall_exit(ql, exit_code, *args, **kw):
    if ql.os.child_processes == True:
        os._exit(0)

    if ql.multithread:
        def _sched_cb_exit(cur_thread):
            ql.log.debug(f"[Thread {cur_thread.get_id()}] Terminated")
            cur_thread.stop()
            cur_thread.exit_code = exit_code
        td = ql.os.thread_management.cur_thread
        ql.emu_stop()
        td.sched_cb = _sched_cb_exit
    else:
        ql.os.exit_code = exit_code
        ql.os.stop()


def ql_syscall_exit_group(ql, exit_code, *args, **kw):
    ql.os.exit_code = exit_code

    if ql.os.child_processes == True:
        os._exit(0)

    ql.os.stop()


def ql_syscall_alarm(ql, alarm_seconds, *args, **kw):
    regreturn = 0
    return regreturn


def ql_syscall_issetugid(ql, *args, **kw):
    UGID = ql.os.uid
    regreturn = UGID
    return regreturn


def ql_syscall_getuid(ql, *args, **kw):
    UID = ql.os.uid
    regreturn = UID
    return regreturn


def ql_syscall_getuid32(ql, *args, **kw):
    UID = ql.os.uid
    regreturn = UID
    return regreturn


def ql_syscall_getgid32(ql, *args, **kw):
    GID = ql.os.gid
    regreturn = GID
    return regreturn


def ql_syscall_geteuid(ql, *args, **kw):
    EUID = ql.os.uid
    regreturn = EUID
    return regreturn


def ql_syscall_getegid(ql, *args, **kw):
    EGID = ql.os.gid
    regreturn = EGID
    return regreturn


def ql_syscall_getgid(ql, *args, **kw):
    GID = ql.os.gid
    regreturn = GID
    return regreturn


def ql_syscall_setgroups(ql, gidsetsize, grouplist, *args, **kw):
    GID = ql.os.gid
    regreturn = GID
    return regreturn


def ql_syscall_setgid(ql, *args, **kw):
    GID = ql.os.gid
    regreturn = GID
    return regreturn


def ql_syscall_setgid32(ql, *args, **kw):
    GID = ql.os.gid
    regreturn = GID
    return regreturn    


def ql_syscall_setuid(ql, *args, **kw):
    UID = ql.os.uid
    regreturn = UID
    return regreturn


def ql_syscall_faccessat(ql, faccessat_dfd, faccessat_filename, faccessat_mode, *args, **kw):

    access_path = ql.mem.string(faccessat_filename)
    real_path = ql.os.transform_to_real_path(access_path)
    relative_path = ql.os.transform_to_relative_path(access_path)

    regreturn = -1
    if os.path.exists(real_path) == False:
        regreturn = -1
    elif stat.S_ISFIFO(Stat(real_path).st_mode):
        regreturn = 0
    else:
        regreturn = -1

    if regreturn == -1:
        ql.log.debug("File Not Found or Skipped: %s" % access_path)
    else:
        ql.log.debug("File Found: %s" % access_path)
    return regreturn


def ql_syscall_lseek(ql, lseek_fd, lseek_ofset, lseek_origin, *args, **kw):
    lseek_ofset = ql.unpacks(ql.pack(lseek_ofset))
    regreturn = 0
    ql.log.debug("lseek(%d, 0x%x, 0x%x) = %d" % (lseek_fd, lseek_ofset, lseek_origin, regreturn))
    try:
        regreturn = ql.os.fd[lseek_fd].lseek(lseek_ofset, lseek_origin)
    except OSError:
        regreturn = -1
    return regreturn


def ql_syscall__llseek(ql, fd, offset_high, offset_low, result, whence, *args, **kw):
    offset = offset_high << 32 | offset_low
    origin = whence
    regreturn = 0
    try:
        ret = ql.os.fd[fd].lseek(offset, origin)
    except OSError:
        regreturn = -1
    #regreturn = 0 if ret >= 0 else -1
    if regreturn == 0:
        ql.mem.write(result, ql.pack64(ret))

    ql.log.debug("_llseek(%d, 0x%x, 0x%x, 0x%x) = %d" % (fd, offset_high, offset_low, origin, regreturn))
    return regreturn


def ql_syscall_brk(ql, brk_input, *args, **kw):
    # current brk_address will be modified if brk_input is not NULL(zero)
    # otherwise, just return current brk_address

    if brk_input != 0:
        new_brk_addr = ((brk_input + 0xfff) // 0x1000) * 0x1000

        if brk_input > ql.loader.brk_address: # increase current brk_address if brk_input is greater
            ql.mem.map(ql.loader.brk_address, new_brk_addr - ql.loader.brk_address, info="[brk]")

        elif brk_input < ql.loader.brk_address: # shrink current bkr_address to brk_input if its smaller
            ql.mem.unmap(new_brk_addr, ql.loader.brk_address - new_brk_addr)

        ql.loader.brk_address = new_brk_addr

    regreturn = ql.loader.brk_address

    ql.log.debug("brk return(0x%x)" % regreturn)
    return regreturn


def ql_syscall_access(ql, access_path, access_mode, *args, **kw):
    path = (ql.mem.string(access_path))

    real_path = ql.os.transform_to_real_path(path)
    relative_path = ql.os.transform_to_relative_path(path)

    if os.path.exists(real_path) == False:
        regreturn = -1
    else:
        regreturn = 0

    ql.log.debug("access(%s, 0x%x) = %d " % (relative_path, access_mode, regreturn))
    if regreturn == 0:
        ql.log.debug("File found: %s" % relative_path)
    else:
        ql.log.debug("No such file or directory")

    return regreturn


def ql_syscall_close(ql, close_fd, *args, **kw):
    regreturn = -1
    if close_fd < 256 and ql.os.fd[close_fd] != 0:
        ql.os.fd[close_fd].close()
        ql.os.fd[close_fd] = 0
        regreturn = 0
    return regreturn


def ql_syscall_pread64(ql, read_fd, read_buf, read_len, read_offt, *args, **kw):
    data = None
    if read_fd < 256 and ql.os.fd[read_fd] != 0:
        try:
            pos = ql.os.fd[read_fd].tell()
            ql.os.fd[read_fd].lseek(read_offt)
            data = ql.os.fd[read_fd].read(read_len)
            ql.os.fd[read_fd].lseek(pos)
            ql.mem.write(read_buf, data)
            regreturn = len(data)
        except:
            regreturn = -1
    else:
        regreturn = -1
    return regreturn


def ql_syscall_read(ql, read_fd, read_buf, read_len, *args, **kw):
    data = None
    if read_fd < 256 and ql.os.fd[read_fd] != 0:
        try:
            data = ql.os.fd[read_fd].read(read_len)
            ql.mem.write(read_buf, data)
            regreturn = len(data)
        except:
            regreturn = -1
    else:
        regreturn = -1

    if data:
        ql.log.debug("read() CONTENT:")
        ql.log.debug("%s" % data)
    return regreturn


def ql_syscall_write(ql, write_fd, write_buf, write_count, *args, **kw):
    regreturn = 0
    buf = None

    try:
        buf = ql.mem.read(write_buf, write_count)
        if buf:
            ql.log.debug("write() CONTENT:")
            ql.log.debug("%s" % buf)

        if hasattr(ql.os.fd[write_fd], "write"):
            
            ql.os.fd[write_fd].write(buf)
        else:
            ql.log.warning("write(%d,%x,%i) failed due to write_fd" % (write_fd, write_buf, write_count, regreturn))
        regreturn = write_count

    except:
        regreturn = -1
        if ql.output in (QL_OUTPUT.DEBUG, QL_OUTPUT.DUMP):
            raise
    #if buf:
    #    ql.log.info(buf.decode(errors='ignore'))
    return regreturn


def ql_syscall_readlink(ql, path_name, path_buff, path_buffsize, *args, **kw):
    pathname = (ql.mem.read(path_name, 0x100).split(b'\x00'))[0]
    pathname = str(pathname, 'utf-8', errors="ignore")

    real_path = ql.os.transform_to_link_path(pathname)
    relative_path = ql.os.transform_to_relative_path(pathname)

    if os.path.exists(real_path) == False:
        regreturn = -1
    elif relative_path == '/proc/self/exe':
        FILEPATH = ql.path
        localpath = os.path.abspath(FILEPATH)
        localpath = bytes(localpath, 'utf-8') + b'\x00'
        ql.mem.write(path_buff, localpath)
        regreturn = (len(localpath)-1)
    else:
        regreturn = 0x0

    ql.log.debug("readlink(%s, 0x%x, 0x%x) = %d" % (relative_path, path_buff, path_buffsize, regreturn))
    return regreturn


def ql_syscall_getcwd(ql, path_buff, path_buffsize, *args, **kw):
    localpath = ql.os.transform_to_relative_path('./')
    localpath = bytes(localpath, 'utf-8') + b'\x00'
    ql.mem.write(path_buff, localpath)
    regreturn = (len(localpath))

    pathname = (ql.mem.read(path_buff, 0x100).split(b'\x00'))[0]
    pathname = str(pathname, 'utf-8', errors="ignore")

    ql.log.debug("getcwd(%s, 0x%x) = %d" % (pathname, path_buffsize, regreturn))
    return regreturn


def ql_syscall_chdir(ql, path_name, *args, **kw):
    regreturn = 0
    pathname = ql.mem.string(path_name)

    real_path = ql.os.transform_to_real_path(pathname)
    relative_path = ql.os.transform_to_relative_path(pathname)

    if os.path.exists(real_path) and os.path.isdir(real_path):
        if ql.os.thread_management != None:
            pass
        else:
            ql.os.current_path = relative_path
        ql.log.debug("chdir(%s) = %d"% (relative_path, regreturn))
    else:
        regreturn = -1
        ql.log.warning("chdir(%s) = %d : Not Found" % (relative_path, regreturn))
    return regreturn


def ql_syscall_readlinkat(ql, readlinkat_dfd, readlinkat_path, readlinkat_buf, readlinkat_bufsiz, *args, **kw):
    pathname = (ql.mem.read(readlinkat_path, 0x100).split(b'\x00'))[0]
    pathname = str(pathname, 'utf-8', errors="ignore")

    real_path = ql.os.transform_to_link_path(pathname)
    relative_path = ql.os.transform_to_relative_path(pathname)

    if os.path.exists(real_path) == False:
        regreturn = -1
    elif relative_path == '/proc/self/exe':
        FILEPATH = ql.path
        localpath = os.path.abspath(FILEPATH)
        localpath = bytes(localpath, 'utf-8') + b'\x00'
        ql.mem.write(readlinkat_buf, localpath)
        regreturn = (len(localpath)-1)
    else:
        regreturn = 0x0
    return regreturn


def ql_syscall_getpid(ql, *args, **kw):
    regreturn= 0x512
    return regreturn


def ql_syscall_getppid(ql, *args, **kw):
    regreturn= 0x1024
    return regreturn


def ql_syscall_vfork(ql, *args, **kw):
    if ql.platform == QL_OS.WINDOWS:
        try:
            pid = Process()
            pid = 0 
        except:
            pid = -1  
    else:
        pid = os.fork()

    if pid == 0:
        ql.os.child_processes = True
        ql.log.debug("vfork(): is this a child process: %r" % (ql.os.child_processes))
        regreturn = 0
    else:
        regreturn = pid

    if ql.os.thread_management != None:
        ql.emu_stop()
    return regreturn


def ql_syscall_setsid(ql, *args, **kw):
    regreturn = os.getpid()
    return regreturn


def ql_syscall_execve(ql, execve_pathname, execve_argv, execve_envp, *args, **kw):
    pathname = ql.mem.string(execve_pathname)
    real_path = ql.os.transform_to_real_path(pathname)
    relative_path = ql.os.transform_to_relative_path(pathname)

    word_size = 8 if (ql.archtype== QL_ARCH.ARM64) or (ql.archtype== QL_ARCH.X8664) else 4
    unpack = ql.unpack64 if (ql.archtype== QL_ARCH.ARM64) or (ql.archtype== QL_ARCH.X8664) else ql.unpack32

    argv = []
    if execve_argv != 0:
        while True:
            argv_addr = unpack(ql.mem.read(execve_argv, word_size))
            if argv_addr == 0:
                break
            argv.append(ql.mem.string(argv_addr))
            execve_argv += word_size

    env = {}
    if execve_envp != 0:
        while True:
            env_addr = unpack(ql.mem.read(execve_envp, word_size))
            if env_addr == 0:
                break
            env_str = ql.mem.string(env_addr)
            idx = env_str.index('=')
            key = env_str[ : idx]
            val = env_str[idx + 1 : ]
            env[key] = val
            execve_envp += word_size

    ql.emu_stop()
    
    ql.log.debug("execve(%s, [%s], [%s])"% (pathname, ', '.join(argv), ', '.join([key + '=' + value for key, value in env.items()])))
    
    ql.loader.argv      = argv
    ql.loader.env       = env
    ql._path             = real_path
    
    ql.mem.map_info     = []
    ql.clear_ql_hooks()
    
    if ql.code:
        return     
    
    # ql._uc               = ql.arch.init_uc
    # ql.os.load()
    # ql.loader.run()
    # ql.run()


def ql_syscall_dup(ql, dup_oldfd, *args, **kw):
    regreturn = -1
    if dup_oldfd in range(0, 256):
        if ql.os.fd[dup_oldfd] != 0:
            newfd = ql.os.fd[dup_oldfd].dup()
            for idx, val in enumerate(ql.os.fd):
                if val == 0:
                    ql.os.fd[idx] = newfd
                    regreturn = idx
                    break

    return regreturn


def ql_syscall_dup2(ql, dup2_oldfd, dup2_newfd, *args, **kw):
    if 0 <= dup2_newfd < 256 and 0 <= dup2_oldfd < 256:
        if ql.os.fd[dup2_oldfd] != 0:
            ql.os.fd[dup2_newfd] = ql.os.fd[dup2_oldfd].dup()
            regreturn = dup2_newfd
        else:
            regreturn = -1
    else:
        regreturn = -1
    return regreturn


def ql_syscall_dup3(ql, dup3_oldfd, dup3_newfd, dup3_flags, null2, null3, null4):
    if 0 <= dup3_newfd < 256 and 0 <= dup3_oldfd < 256:
        if ql.os.fd[dup3_oldfd] != 0:
            ql.os.fd[dup3_newfd] = ql.os.fd[dup3_oldfd].dup()
            regreturn = dup3_newfd
        else:
            regreturn = -1
    else:
        regreturn = -1
    return regreturn

def ql_syscall_set_tid_address(ql, set_tid_address_tidptr, *args, **kw):
    if ql.os.thread_management == None:
        regreturn = os.getpid()
    else:
        ql.os.thread_management.cur_thread.set_clear_child_tid_addr(set_tid_address_tidptr)
        regreturn = ql.os.thread_management.cur_thread.id
    return regreturn


def ql_syscall_pipe(ql, pipe_pipefd, *args, **kw):
    rd, wd = ql_pipe.open()

    idx1 = -1
    for i in range(256):
        if ql.os.fd[i] == 0:
            idx1 = i
            break
    if idx1 == -1:
        regreturn = -1
    else:
        idx2 = -1
        for i in range(256):
            if ql.os.fd[i] == 0 and i != idx1:
                idx2 = i
                break
        if idx2 == -1:
            regreturn = -1
        else:
            ql.os.fd[idx1] = rd
            ql.os.fd[idx2] = wd
            if ql.archtype== QL_ARCH.MIPS:
                ql.reg.v1 = idx2
                regreturn = idx1
            else:
                ql.mem.write(pipe_pipefd, ql.pack32(idx1) + ql.pack32(idx2))
                regreturn = 0

    ql.log.debug("pipe(%x, [%d, %d]) = %d" % (pipe_pipefd, idx1, idx2, regreturn))
    return regreturn


def ql_syscall_nice(ql, nice_inc, *args, **kw):
    regreturn = 0
    return regreturn


def ql_syscall_truncate(ql, path, length, *args, **kw):
    path = ql.mem.string(path)
    real_path = ql.os.transform_to_real_path(path)
    st_size = Stat(real_path).st_size

    try:
        if st_size >= length:
            os.truncate(real_path, length)

        else:
            padding = (length - st_size)
            with open(real_path, 'a+b') as fd:
                fd.write(b'\x00'*padding)

        regreturn = 0
    except:
        regreturn = -1

    ql.log.debug('truncate(%s, 0x%x) = %d' % (path, length, regreturn))
    return regreturn


def ql_syscall_ftruncate(ql, ftrunc_fd, ftrunc_length, *args, **kw):
    real_path = ql.os.fd[ftrunc_fd].name
    st_size = Stat(real_path).st_size

    try:
        if st_size >= ftrunc_length:
            os.truncate(real_path, ftrunc_length)

        else:
            padding = (ftrunc_length - st_size)
            with open(real_path, 'a+b') as fd:
                fd.write(b'\x00'*padding)

        regreturn = 0
    except:
        regreturn = -1

    ql.log.debug("ftruncate(%d, 0x%x) = %d" % (ftrunc_fd, ftrunc_length, regreturn))
    return regreturn


def ql_syscall_unlink(ql, unlink_pathname, *args, **kw):
    pathname = ql.mem.string(unlink_pathname)
    real_path = ql.os.transform_to_real_path(pathname)
    opened_fds = [getattr(ql.os.fd[i], 'name', None) for i in range(256) if ql.os.fd[i] != 0]
    path = pathlib.Path(real_path)

    if any((real_path not in opened_fds, path.is_block_device(), path.is_fifo(), path.is_socket(), path.is_symlink())):
        try:
            os.unlink(real_path)
            regreturn = 0
        except FileNotFoundError:
            ql.log.debug('No such file or directory')
            regreturn = -1
        except:
            regreturn = -1
    else:
        regreturn = -1

    ql.log.debug("unlink(%s) = %d" % (pathname, regreturn))
    return regreturn


def ql_syscall_unlinkat(ql, dirfd, pathname, flag, *args, **kw):
    #FIXME dirfd(relative path) not implement.
    file_path = ql.mem.string(pathname)
    real_path = ql.os.transform_to_real_path(file_path)
    ql.log.debug("unlinkat(%d, %s, 0%o)" % (dirfd, real_path, flag))
    try:
        os.unlink(real_path)
        return 0
    except FileNotFoundError:
        ql.log.debug("No such file or directory")
        return -1
    except:
        return -1


def ql_syscall_getdents(ql, fd, dirp, count, *args, **kw):
    # TODO: not sure what is the meaning of d_off, should not be 0x0
    # but works for the example code from linux manual.
    def _type_mapping(ent):
        methods_constants_d = {'is_fifo': 0x1, 'is_char_device': 0x2, 'is_dir': 0x4, 'is_block_device': 0x6,
                                'is_file': 0x8, 'is_symlink': 0xa, 'is_socket': 0xc}
        ent_p = pathlib.Path(ent.path) if isinstance(ent, os.DirEntry) else ent

        for method, constant in methods_constants_d.items():
            if getattr(ent_p, method, None)():
                t = constant
                break
        else:
            t = 0x0 # DT_UNKNOWN

        return bytes([t])

    if ql.os.fd[fd].tell() == 0:
        n = ql.archbit // 8
        total_size = 0
        results = os.scandir(ql.os.fd[fd].name)
        _ent_count = 0

        for result in itertools.chain((pathlib.Path('.'), pathlib.Path('..')), results): # chain speical directories with the results
            d_ino = result.inode() if isinstance(result, os.DirEntry) else result.stat().st_ino
            d_off = 0x0
            d_name = (result.name if isinstance(result, os.DirEntry) else result._str).encode() + b'\x00'
            d_type = _type_mapping(result)
            d_reclen = len(d_name) + n*2 + 3

            ql.mem.write(dirp, ql.pack(d_ino))
            ql.mem.write(dirp+n, ql.pack(d_off))
            ql.mem.write(dirp+n*2, ql.pack16(d_reclen))
            ql.mem.write(dirp+n*2+2, d_name)
            ql.mem.write(dirp+n*2+2+len(d_name), d_type)

            dirp += d_reclen
            total_size += d_reclen
            _ent_count += 1

        regreturn = total_size
        ql.os.fd[fd].lseek(0, os.SEEK_END) # mark as end of file for dir_fd
    else:
        _ent_count = 0
        regreturn = 0

    ql.log.debug("getdents(%d, /* %d entries */, 0x%x) = %d" % (fd, _ent_count, count, regreturn))
    return regreturn




#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.os.const import *
from ..fncc import *
from ..ProcessorBind import *
from ..UefiBaseType import *
from ..const import *

# @see: MdePkg/Include/Protocol/MmCpu.h
class EFI_SMM_SAVE_STATE_REGISTER(ENUM_UC):
	_members_ = {
		# note: members names were shorten from 'EFI_SMM_SAVE_STATE_REGISTER_regname' to just 'regname' 
		'GDTBASE'	: 4,
		'IDTBASE'	: 5,
		'LDTBASE'	: 6,
		'GDTLIMIT'	: 7,
		'IDTLIMIT'	: 8,
		'LDTLIMIT'	: 9,
		'LDTINFO'	: 10,

		'ES'		: 20,
		'CS'		: 21,
		'SS'		: 22,
		'DS'		: 23,
		'FS'		: 24,
		'GS'		: 25,
		'LDTR_SEL'	: 26,
		'TR_SEL'	: 27,
		'DR7'		: 28,
		'DR6'		: 29,
		'R8'		: 30,
		'R9'		: 31,
		'R10'		: 32,
		'R11'		: 33,
		'R12'		: 34,
		'R13'		: 35,
		'R14'		: 36,
		'R15'		: 37,
		'RAX'		: 38,
		'RBX'		: 39,
		'RCX'		: 40,
		'RDX'		: 41,
		'RSP'		: 42,
		'RBP'		: 43,
		'RSI'		: 44,
		'RDI'		: 45,
		'RIP'		: 46,

		'RFLAGS'	: 51,
		'CR0'		: 52,
		'CR3'		: 53,
		'CR4'		: 54,

		'FCW'		: 256,
		'FSW'		: 257,
		'FTW'		: 258,
		'OPCODE'	: 259,
		'FP_EIP'	: 260,
		'FP_CS'		: 261,
		'DATAOFFSET': 262,
		'FP_DS'		: 263,
		'MM0'		: 264,
		'MM1'		: 265,
		'MM2'		: 266,
		'MM3'		: 267,
		'MM4'		: 268,
		'MM5'		: 269,
		'MM6'		: 270,
		'MM7'		: 271,
		'XMM0'		: 272,
		'XMM1'		: 273,
		'XMM2'		: 274,
		'XMM3'		: 275,
		'XMM4'		: 276,
		'XMM5'		: 277,
		'XMM6'		: 278,
		'XMM7'		: 279,
		'XMM8'		: 280,
		'XMM9'		: 281,
		'XMM10'		: 282,
		'XMM11'		: 283,
		'XMM12'		: 284,
		'XMM13'		: 285,
		'XMM14'		: 286,
		'XMM15'		: 287,

		'IO'			: 512,
		'LMA'			: 513,
		'PROCESSOR_ID'	: 514
	}

# EFI_SUCCESS			The register was written from Save State
# EFI_NOT_FOUND			The register is not defined for the Save State of Processor
# EFI_INVALID_PARAMETER	ProcessorIndex or Width is not correct

@dxeapi(params = {
	"This"		: POINTER,	# EFI_SMM_CPU_PROTOCOL
	"Width"		: UINT,		# UINTN
	"Register"	: INT,		# EFI_SMM_SAVE_STATE_REGISTER
	"CpuIndex"	: UINT,		# UINTN
	"Buffer"	: POINTER	# PTR(VOID))
})
def hook_SmmReadSaveState(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params = {
	"This"		: POINTER,	# EFI_SMM_CPU_PROTOCOL
	"Width"		: UINT,		# UINTN
	"Register"	: INT,		# EFI_SMM_SAVE_STATE_REGISTER
	"CpuIndex"	: UINT,		# UINTN
	"Buffer"	: POINTER	# PTR(VOID))
})
def hook_SmmWriteSaveState(ql, address, params):
	return EFI_SUCCESS

class EFI_SMM_CPU_PROTOCOL(STRUCT):
	EFI_SMM_CPU_PROTOCOL = STRUCT

	_fields_ = [
		('SmmReadSaveState',	FUNCPTR(PTR(EFI_SMM_CPU_PROTOCOL), UINTN, EFI_SMM_SAVE_STATE_REGISTER, UINTN, PTR(VOID))),
		('SmmWriteSaveState',	FUNCPTR(PTR(EFI_SMM_CPU_PROTOCOL), UINTN, EFI_SMM_SAVE_STATE_REGISTER, UINTN, PTR(VOID)))
	]

descriptor = {
	"guid" : "eb346b97-975f-4a9f-8b22-f8e92bb3d569",
	"struct" : EFI_SMM_CPU_PROTOCOL,
	"fields" : (
		("SmmReadSaveState",	hook_SmmReadSaveState),
		("SmmWriteSaveState",	hook_SmmWriteSaveState)
	)
}



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import json, os, sys

from Registry import Registry

from qiling.os.windows.const import *
from qiling.exception import *
from qiling.const import *


# Registry Manager reads data from two places
# 1. config.json
#       if you want to modify the registry key/value, you can modify config.json
#       If there is a registry entry in config.json that needs to be read, 
#       Registry Manager will read from config.json first.
# 2. windows hive files

# Registry Manager will only write registry changes to config.json 
# and will not modify the hive file.


class RegistryManager:
    def __init__(self, ql, hive=None):
        self.ql = ql
        self.log_registry_dir = self.ql.rootfs
        
        if self.log_registry_dir == None:
            self.log_registry_dir = "qlog"

        self.registry_diff = self.ql.targetname + "_diff.json"
        self.regdiff = os.path.join(self.log_registry_dir, "registry", self.registry_diff)    

        # hive dir
        if hive:
            self.hive = hive
        else:
            self.hive = os.path.join(ql.rootfs, "Windows", "registry")
            ql.log.debug("Windows Registry PATH: %s" % self.hive)
            if not os.path.exists(self.hive) and not self.ql.code:
                raise QlErrorFileNotFound(f"Error: Registry files not found in '{self.hive}'!")

        if not os.path.exists(self.regdiff):
            self.registry_config = {}
            try:
                os.makedirs(os.path.join(self.log_registry_dir, "registry"), 0o755)
            except Exception:
                pass
        else:
            # read config
            # use registry config first
            self.f_config = open(self.regdiff, "rb")
            data = self.f_config.read()
            if data == b"":
                self.registry_config = {}
                self.f_config.close()
            else:
                try:
                    self.registry_config = json.loads(data)
                except json.decoder.JSONDecodeError:
                    raise QlErrorJsonDecode("Windows Registry JSON decode error")
                finally:
                    self.f_config.close()

        # hkey local system
        self.hklm = {}
        try:
            self.hklm['SECURITY'] = Registry.Registry(os.path.join(self.hive, 'SECURITY'))
            self.hklm['SAM'] = Registry.Registry(os.path.join(self.hive, 'SAM'))
            self.hklm['SOFTWARE'] = Registry.Registry(os.path.join(self.hive, 'SOFTWARE'))
            self.hklm['SYSTEM'] = Registry.Registry(os.path.join(self.hive, 'SYSTEM'))
            self.hklm['HARDWARE'] = Registry.Registry(os.path.join(self.hive, 'HARDWARE'))
            # hkey current user
            self.hkcu = Registry.Registry(os.path.join(self.hive, 'NTUSER.DAT'))
        except FileNotFoundError:
            if not ql.code:
                QlErrorFileNotFound("WARNING: Registry files not found!")
        except Exception:
            if not ql.code:
                QlErrorFileNotFound("WARNING: Registry files format error")
        self.accessed = {}

    def exists(self, key):
        if key in self.regdiff:
            return True
        keys = key.split("\\")
        self.access(key)
        try:
            if keys[0] == "HKEY_LOCAL_MACHINE":
                reg = self.hklm[keys[1]]
                sub = "\\".join(keys[2:])
                data = reg.open(sub)
            elif keys[0] == "HKEY_CURRENT_USER":
                reg = self.hkcu
                sub = "\\".join(keys[1:])
                data = reg.open(sub)
            else:
                raise QlErrorNotImplemented("Windows Registry %s not implemented" % (keys[0]))
        except Exception:
            return False

        return True

    def read(self, key, subkey, reg_type):
        # of the key, the subkey is the value checked

        # read reg conf first
        if key in self.regdiff and subkey in self.regdiff[key]:
            if self.regdiff[key][subkey].type in REG_TYPES:
                return REG_TYPES[self.regdiff[key][subkey].type], self.regdiff[key][subkey].value
            else:
                raise QlErrorNotImplemented(
                    "Windows Registry Type %s not implemented" % self.regdiff[key][subkey].type)

        # read hive
        reg = None
        data = None
        keys = key.split('\\')
        try:
            if keys[0] == "HKEY_LOCAL_MACHINE":
                reg = self.hklm[keys[1]]
                sub = "\\".join(keys[2:])
                data = reg.open(sub)
            elif keys[0] == "HKEY_CURRENT_USER":
                reg = self.hkcu
                sub = "\\".join(keys[1:])
                data = reg.open(sub)
            else:
                raise QlErrorNotImplemented("Windows Registry %s not implemented" % (keys[0]))

            for value in data.values():
                if value.name() == subkey and (reg_type == Registry.RegNone or
                                               value.value_type() == reg_type):

                    self.access(key, value_name=subkey, value=value.value(), type=value.value_type())
                    return value.value_type(), value.value()

        except Registry.RegistryKeyNotFoundException:
            pass

        self.access(key, value_name=subkey, value=None, type=None)

        return None, None

    def access(self, key, value_name=None, value=None, type=None):
        if value_name is None:
            if key not in self.accessed:
                self.accessed[key] = []
        else:
            self.accessed[key].append({
                "value_name": value_name,
                "value": value,
                "type": type,
                "position": self.ql.os.syscalls_counter
            })
            # we don't have to increase the counter since we are technically inside a hook

    def create(self, key):
        self.registry_config[key] = dict()

    def write(self, key, subkey, reg_type, data):
        if key not in self.registry_config:
            self.create(key)
        # write registry changes to config.json
        self.registry_config[key][subkey] = {
            "type": REG_TYPES[reg_type],
            "value": data
        }

    def delete(self, key, subkey):
        del self.registry_config[key][subkey]

    @staticmethod
    def _encode_binary_value(data):
        # bytes(hex(data), 'ascii')
        # TODO
        pass

    def write_reg_value_into_mem(self, reg_value, reg_type, address):
        length = 0
        # string
        if reg_type == Registry.RegSZ or reg_type == Registry.RegExpandSZ:
            self.ql.mem.write(address, bytes(reg_value, "utf-16le") + b"\x00")
            length = len(reg_value)
        elif reg_type == Registry.RegBin:
            # you can set REG_BINARY like '\x00\x01\x02' in config.json
            if type(reg_value) == str:
                self.ql.mem.write(address, bytes(reg_value))
                length = len(reg_value)
            else:
                raise QlErrorNotImplemented("Windows Registry Type not implemented")
        elif reg_type == Registry.RegDWord:
            data = self.ql.pack32(reg_value)
            self.ql.mem.write(address, data)
            length = len(data)
        elif reg_type == Registry.RegQWord:
            data = self.ql.pack64(reg_value)
            self.ql.mem.write(address, data)
            length = len(data)
        else:
            raise QlErrorNotImplemented(
                "Windows Registry Type write to memory %s not implemented" % (REG_TYPES[reg_type]))

        return length

    def save(self):
        # write registry config to config file
        if self.registry_config and len(self.registry_config) != 0:
            with open(self.regdiff, "wb") as f:
                f.write(bytes(json.dumps(self.registry_config), "utf-8"))



# class for colorful prints
class color:
   CYAN      = '\033[96m'
   PURPLE    = '\033[95m'
   BLUE      = '\033[94m'
   YELLOW    = '\033[93m'
   GREEN     = '\033[92m'
   RED       = '\033[91m'
   DARKGRAY  = '\033[90m'
   WHITE     = '\033[48m'
   DARKCYAN  = '\033[36m'
   BLACK     = '\033[35m'
   UNDERLINE = '\033[4m'
   BOLD      = '\033[1m'
   END       = '\033[0m'


FORMAT_LETTER = {
        "o", # octal
        "x", # hex
        "d", # decimal
        "u", # unsigned decimal
        "t", # binary
        "f", # float
        "a", # address
        "i", # instruction
        "c", # char
        "s", # string
        "z", # hex, zero padded on the left
        }


SIZE_LETTER = {
    "b": 1, # 1-byte, byte
    "h": 2, # 2-byte, halfword
    "w": 4, # 4-byte, word
    "g": 8, # 8-byte, giant
    }



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *
from qiling.os.windows.structs import *


dllname = 'kernel32_dll'


# void ExitProcess(
#   UINT uExitCode
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'UINT': 'DWORD'})
def hook_ExitProcess(ql, address, params):
    ql.emu_stop()
    ql.os.PE_RUN = False


def _GetStartupInfo(ql, address, params):
    startup_info = StartupInfo(ql, 0xc3c930, 0, 0, 0, 0x64, 0x64, 0x84, 0x80, 0xff, 0x40, 0x1, STD_INPUT_HANDLE,
                               STD_OUTPUT_HANDLE, STD_ERROR_HANDLE)

    pointer = params["lpStartupInfo"]
    startup_info.write(pointer)
    return 0


# VOID WINAPI GetStartupInfoA(
#   _Out_ LPSTARTUPINFO lpStartupInfo
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetStartupInfoA(ql, address, params):
    return _GetStartupInfo(ql, address, params)


# VOID WINAPI GetStartupInfoW(
#   _Out_ LPSTARTUPINFO lpStartupInfo
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetStartupInfoW(ql, address, params):
    # The struct for the W version uses LPWSTRING, but i think is the same in this context
    return _GetStartupInfo(ql, address, params)


# DWORD TlsAlloc();
@winsdkapi(cc=STDCALL)
def hook_TlsAlloc(ql, address, params):
    idx = ql.os.thread_manager.cur_thread.tls_index
    ql.os.thread_manager.cur_thread.tls_index += 1
    ql.os.thread_manager.cur_thread.tls[idx] = 0
    return idx


# DWORD TlsFree(
#  DWORD dwTlsIndex
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={"dwTlsIndex": UINT})
def hook_TlsFree(ql, address, params):
    idx = params['dwTlsIndex']
    if idx not in ql.os.thread_manager.cur_thread.tls:
        ql.os.last_error = 0x57  # (ERROR_INVALID_PARAMETER)
        return 0
    else:
        del (ql.os.thread_manager.cur_thread.tls[idx])
        return 1


# LPVOID TlsGetValue(
#  DWORD dwTlsIndex
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={"dwTlsIndex": UINT})
def hook_TlsGetValue(ql, address, params):
    idx = params['dwTlsIndex']
    if idx not in ql.os.thread_manager.cur_thread.tls:
        ql.os.last_error = 0x57  # (ERROR_INVALID_PARAMETER)
        return 0
    else:
        # api explicity clears last error on success:
        # https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-tlsgetvalue
        ql.os.last_error = 0
        return ql.os.thread_manager.cur_thread.tls[idx]


# LPVOID TlsSetValue(
#  DWORD dwTlsIndex
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={"dwTlsIndex": UINT, "lpTlsValue": POINTER})
def hook_TlsSetValue(ql, address, params):
    idx = params['dwTlsIndex']
    if idx not in ql.os.thread_manager.cur_thread.tls:
        ql.os.last_error = 0x57  # (ERROR_INVALID_PARAMETER)
        return 0
    else:
        ql.os.thread_manager.cur_thread.tls[idx] = params['lpTlsValue']
        return 1


# DWORD GetCurrentThreadId(
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetCurrentThreadId(ql, address, params):
    ret = ql.os.thread_manager.cur_thread.id
    return ret


# DWORD GetCurrentProcessId(
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetCurrentProcessId(ql, address, params):
    # Let's return a valid value
    return ql.os.profile.getint("KERNEL", "pid")


# BOOL IsProcessorFeaturePresent(
#   DWORD ProcessorFeature
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_IsProcessorFeaturePresent(ql, address, params):
    feature = params["ProcessorFeature"]
    if feature == PF_XSAVE_ENABLED:
        # it seems like unicorn can't recognize the instruction
        return 0
    else:
        return 1


# HANDLE CreateThread(
#   LPSECURITY_ATTRIBUTES   lpThreadAttributes,
#   SIZE_T                  dwStackSize,
#   LPTHREAD_START_ROUTINE  lpStartAddress,
#   __drv_aliasesMem LPVOID lpParameter,
#   DWORD                   dwCreationFlags,
#   LPDWORD                 lpThreadId
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CreateThread(ql, address, params):
    CREATE_RUN = 0
    CREATE_SUSPENDED = 0x00000004

    ret = 0
    lpThreadAttributes = params["lpThreadAttributes"]
    dwStackSize = params["dwStackSize"]
    lpStartAddress = params["lpStartAddress"]
    lpParameter = params["lpParameter"]
    dwCreationFlags = params["dwCreationFlags"]
    lpThreadId = params["lpThreadId"]

    # new thread obj
    new_thread = QlWindowsThread(ql)

    if dwCreationFlags & CREATE_SUSPENDED == CREATE_SUSPENDED:
        thread_status = QlWindowsThread.READY
    else:
        thread_status = QlWindowsThread.RUNNING

    # create new thread
    thread_id = new_thread.create(
        lpStartAddress,
        lpParameter,
        thread_status
    )

    # append the new thread to ThreadManager
    ql.os.thread_manager.append(new_thread)

    # create thread handle
    new_handle = Handle(obj=new_thread)
    ql.os.handle_manager.append(new_handle)
    ret = new_handle.id

    # set lpThreadId
    # FIXME: Temporary fix for the crash
    # if lpThreadId != 0:
    # ql.mem.write(lpThreadId, ql.pack(thread_id))

    # set thread handle
    return ret


# HANDLE GetCurrentProcess(
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetCurrentProcess(ql, address, params):
    return ql.os.profile.getint("KERNEL", "pid")


# BOOL TerminateProcess(
#   HANDLE hProcess,
#   UINT   uExitCode
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_TerminateProcess(ql, address, params):
    # Samples will try to kill other process! We don't want to always stop!
    process = params["hProcess"]
    if process == ql.os.profile.getint("KERNEL", "pid"):  # or process == ql.os.image_address:
        ql.emu_stop()
        ql.os.PE_RUN = False
    ret = 1
    return ret


# HANDLE GetCurrentThread();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetCurrentThread(ql, address, params):
    ret = ql.os.thread_manager.cur_thread.id
    return ret


# HANDLE OpenProcess(
#   DWORD dwDesiredAccess,
#   BOOL  bInheritHandle,
#   DWORD dwProcessId
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'BOOL': 'HANDLE'})
def hook_OpenProcess(ql, address, params):
    proc = params["dwProcessId"]
    # If the specified process is the System Process (0x00000000),
    # the function fails and the last error code is ERROR_INVALID_PARAMETER
    if proc == 0:
        ql.os.last_error = ERROR_INVALID_PARAMETER
        return 0
    #  If the specified process is the Idle process or one of the CSRSS processes, this function fails
    #  and the last error code is ERROR_ACCESS_DENIED because their access restrictions prevent user-level code
    #  from opening them.
    if proc == ql.profile.getint("PROCESSES", "csrss.exe"):
        ql.os.last_error = ERROR_ACCESS_DENIED
        return 0
    return 0xD10C


# BOOL OpenProcessToken(
#   HANDLE  ProcessHandle,
#   DWORD   DesiredAccess,
#   PHANDLE TokenHandle
# );
@winsdkapi(cc=STDCALL, dllname=dllname,
           replace_params={"ProcessHandle": HANDLE, "DesiredAccess": DWORD, "TokenHandle": POINTER})
def hook_OpenProcessToken(ql, address, params):
    token_pointer = params["TokenHandle"]
    token = Token(ql)
    new_handle = Handle(obj=token)
    ql.os.handle_manager.append(new_handle)
    ql.mem.write(token_pointer, ql.pack(new_handle.id))
    return 1


# BOOL GetThreadContext(
#   HANDLE    hThread,
#   LPCONTEXT lpContext
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetThreadContext(ql, address, params):
    return 1


# BOOL OpenThreadToken(
#   HANDLE  ThreadHandle,
#   DWORD   DesiredAccess,
#   BOOL    OpenAsSelf,
#   PHANDLE TokenHandle
# );
@winsdkapi(cc=STDCALL, dllname=dllname,
           replace_params={"ThreadHandle": HANDLE, "DesiredAccess": DWORD, "OpenAsSelf": BOOL, "TokenHandle": POINTER})
def hook_OpenThreadToken(ql, address, params):
    token_pointer = params["TokenHandle"]
    token = Token(ql)
    new_handle = Handle(obj=token)
    ql.os.handle_manager.append(new_handle)
    ql.mem.write(token_pointer, ql.pack(new_handle.id))
    return 1


# BOOL GetThreadTimes(
#   HANDLE     hThread,
#   LPFILETIME lpCreationTime,
#   LPFILETIME lpExitTime,
#   LPFILETIME lpKernelTime,
#   LPFILETIME lpUserTime
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetThreadTimes(ql, address, params):
    return 1



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

# gdbserver --remote-debug 0.0.0.0:9999 /path/to binary
# documentation: according to https://sourceware.org/gdb/current/onlinedocs/gdb/Remote-Protocol.html#Remote-Protocol

from unicorn import *

import struct, os, re, socket
from binascii import unhexlify

from .utils import QlGdbUtils
from qiling.const import *
from qiling.utils import *
from qiling.debugger import QlDebugger
from qiling.arch.x86_const import reg_map_16 as x86_reg_map_16
from qiling.arch.x86_const import reg_map_32 as x86_reg_map_32
from qiling.arch.x86_const import reg_map_64 as x86_reg_map_64
from qiling.arch.x86_const import reg_map_misc as x86_reg_map_misc
from qiling.arch.x86_const import reg_map_st as x86_reg_map_st
from qiling.arch.arm_const import reg_map as arm_reg_map
from qiling.arch.arm64_const import reg_map as arm64_reg_map
from qiling.arch.mips_const import reg_map as mips_reg_map

GDB_SIGNAL_INT  = 2
GDB_SIGNAL_SEGV = 11
GDB_SIGNAL_GILL = 4
GDB_SIGNAL_STOP = 17
GDB_SIGNAL_TRAP = 5
GDB_SIGNAL_BUS  = 10


class QlGdb(QlDebugger, object):
    """docstring for Debugsession"""
    def __init__(self, ql, ip, port):
        super(QlGdb, self).__init__(ql)
        self.ql             = ql
        self.last_pkt       = None
        self.exe_abspath    = (os.path.abspath(self.ql.argv[0]))
        self.rootfs_abspath = (os.path.abspath(self.ql.rootfs)) 
        self.gdb            = QlGdbUtils()

        if ip == None:
            ip = '127.0.0.1'
        
        if port == None:
           port = 9999
        else:
            port = int(port)

        self.ip = ip
        self.port = port

        if ql.code:
            load_address = ql.os.entry_point
            exit_point = load_address + len(ql.code)
        else:
            load_address = ql.loader.load_address
            exit_point = load_address + os.path.getsize(ql.path)

        self.gdb.initialize(self.ql, exit_point=exit_point, mappings=[(hex(load_address))])
        
        if self.ql.ostype in (QL_OS.LINUX, QL_OS.FREEBSD) and not self.ql.code:
            self.entry_point = self.ql.os.elf_entry
        else:
            self.entry_point = self.ql.os.entry_point
           
        self.gdb.bp_insert(self.entry_point)

        #Setup register tables, order of tables is important
        self.tables = {
            QL_ARCH.A8086   : list({**x86_reg_map_16, **x86_reg_map_misc}.keys()),
            QL_ARCH.X86     : list({**x86_reg_map_32, **x86_reg_map_misc, **x86_reg_map_st}.keys()),
            QL_ARCH.X8664   : list({**x86_reg_map_64, **x86_reg_map_misc, **x86_reg_map_st}.keys()),
            QL_ARCH.ARM     : list({**arm_reg_map}.keys()),
            QL_ARCH.ARM64   : list({**arm64_reg_map}.keys()),
            QL_ARCH.MIPS    : list({**mips_reg_map}.keys()),
        }

    def addr_to_str(self, addr, short=False, endian="big"):
        if self.ql.archbit == 64 and short == False:
            addr = (hex(int.from_bytes(self.ql.pack64(addr), byteorder=endian)))
            addr = '{:0>16}'.format(addr[2:])
        elif self.ql.archbit == 32 or short == True:
            addr = (hex(int.from_bytes(self.ql.pack32(addr), byteorder=endian)))
            addr = ('{:0>8}'.format(addr[2:]))
        elif self.ql.archbit == 16 or short == True:
            addr = (hex(int.from_bytes(self.ql.pack32(addr), byteorder=endian)))
            addr = ('{:0>8}'.format(addr[2:]))            
        addr = str(addr)    
        return addr

    def bin_to_escstr(self, rawbin):
        rawbin_escape = ""

        def incomplete_hex_check(hexchar):
            if len(hexchar) == 1:
                hexchar = "0" + hexchar
            return hexchar

        for a in rawbin:

            # The binary data representation uses 7d (ASCII ‘}’) as an escape character. 
            # Any escaped byte is transmitted as the escape character followed by the original character XORed with 0x20. 
            # For example, the byte 0x7d would be transmitted as the two bytes 0x7d 0x5d. The bytes 0x23 (ASCII ‘#’), 0x24 (ASCII ‘$’), and 0x7d (ASCII ‘}’) 
            # must always be escaped. Responses sent by the stub must also escape 0x2a (ASCII ‘*’), 
            # so that it is not interpreted as the start of a run-length encoded sequence (described next).

            if a in (42,35,36, 125):
                a = a ^ 0x20
                a = (str(hex(a)[2:]))
                a = incomplete_hex_check(a)
                a = str("7d%s" % a)
            else:
                a = (str(hex(a)[2:]))
                a = incomplete_hex_check(a)

            rawbin_escape += a

        return unhexlify(rawbin_escape)

    def setup_server(self):
        self.ql.log.info("gdb> Listening on %s:%u" % (self.ip, self.port))

        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind((self.ip, self.port))
        sock.listen(1)
        clientsocket, addr = sock.accept()

        self.sock           = sock
        self.clientsocket   = clientsocket
        self.netin          = clientsocket.makefile('r')
        self.netout         = clientsocket.makefile('w')

    def close(self):
        self.netin.close()
        self.netout.close()
        self.clientsocket.close()
        self.sock.close()

    def run(self):
        self.setup_server()

        while self.receive() == 'Good':
            pkt = self.last_pkt
            self.send_raw('+')

            def handle_qmark(subcmd):
                def gdbqmark_converter(arch):
                    """
                    MIPS32_EL : gdbserver response ("$T051d:00e7ff7f;25:40ccfc77;#65")
                    MIPS32_EB : gdbserver response ("$T051d:7fff6dc0;25:77fc4880;thread:28fa;core:0;");
                    ARM64: gdbserver response "$T051d:0*,;1f:80f6f*"ff0* ;20:c02cfdb7f* 0* ;thread:p1f9.1f9;core:0;#56");
                    ARM: gdbserver $T050b:0*"00;0d:e0f6ffbe;0f:8079fdb6;#ae"
                    """
                    adapter = {
                        QL_ARCH.A8086        : [ 0x05, 0x04, 0x08 ],
                        QL_ARCH.X86          : [ 0x05, 0x04, 0x08 ],
                        QL_ARCH.X8664        : [ 0x06, 0x07, 0x10 ],
                        QL_ARCH.MIPS         : [ 0x1d, 0x00, 0x25 ],        
                        QL_ARCH.ARM          : [ 0x0b, 0x0d, 0x0f ],
                        QL_ARCH.ARM64        : [ 0x1d, 0xf1, 0x20 ]
                        }
                    return adapter.get(arch)

                idhex, spid, pcid  = gdbqmark_converter(self.ql.archtype)  
                sp          = self.addr_to_str(self.ql.reg.arch_sp)
                pc          = self.addr_to_str(self.ql.reg.arch_pc)
                nullfill    = "0" * int(self.ql.archbit / 4)

                if self.ql.archtype== QL_ARCH.MIPS:
                    if self.ql.archendian == QL_ENDIAN.EB:
                        sp = self.addr_to_str(self.ql.reg.arch_sp, endian ="little")
                        pc = self.addr_to_str(self.ql.reg.arch_pc, endian ="little")
                    self.send('T%.2x%.2x:%s;%.2x:%s;' %(GDB_SIGNAL_TRAP, idhex, sp, pcid, pc))
                else:    
                    self.send('T%.2x%.2x:%s;%.2x:%s;%.2x:%s;' %(GDB_SIGNAL_TRAP, idhex, nullfill, spid, sp, pcid, pc))


            def handle_c(subcmd):
                self.gdb.resume_emu(self.ql.reg.arch_pc)

                if self.gdb.bp_list == [self.entry_point]:
                    self.send("W00")
                else:
                    self.send(('S%.2x' % GDB_SIGNAL_TRAP))


            handle_C = handle_c


            def handle_g(subcmd):
                s = ''

                if self.ql.archtype== QL_ARCH.A8086:
                    for reg in self.tables[QL_ARCH.A8086][:16]:
                        r = self.ql.reg.read(reg)
                        tmp = self.addr_to_str(r)
                        s += tmp

                elif self.ql.archtype== QL_ARCH.X86:
                    for reg in self.tables[QL_ARCH.X86][:16]:
                        r = self.ql.reg.read(reg)
                        tmp = self.addr_to_str(r)
                        s += tmp

                elif self.ql.archtype== QL_ARCH.X8664:
                    for reg in self.tables[QL_ARCH.X8664][:24]:
                        r = self.ql.reg.read(reg)
                        if self.ql.reg.bit(reg) == 64:
                            tmp = self.addr_to_str(r)
                        elif self.ql.reg.bit(reg) == 32:
                            tmp = self.addr_to_str(r, short = True)
                        s += tmp
                
                elif self.ql.archtype == QL_ARCH.ARM:
                    mode = self.ql.arch.check_thumb()
                    
                    for reg in self.tables[QL_ARCH.ARM][:16]:
                        r = self.ql.reg.read(reg)
                        if mode == UC_MODE_THUMB and reg == "pc":
                            r += 1
                        elif mode != UC_MODE_THUMB and reg == "pc":
                            r += 4
                        tmp = self.addr_to_str(r)
                        s += tmp

                elif self.ql.archtype == QL_ARCH.ARM64:
                    for reg in self.tables[QL_ARCH.ARM64][:33]:
                        r = self.ql.reg.read(reg)
                        tmp = self.addr_to_str(r)
                        s += tmp

                elif self.ql.archtype == QL_ARCH.MIPS:
                    for reg in self.tables[QL_ARCH.MIPS][:38]:
                        r = self.ql.reg.read(reg)
                        if self.ql.archendian == QL_ENDIAN.EB:
                            tmp = self.addr_to_str(r, endian ="little")
                        else:
                            tmp = self.addr_to_str(r)    
                        s += tmp

                self.send(s)


            def handle_G(subcmd):
                count = 0

                if self.ql.archtype == QL_ARCH.A8086:
                    for i in range(0, len(subcmd), 8):
                        reg_data = subcmd[i:i+7]
                        reg_data = int(reg_data, 16)
                        self.ql.reg.write(self.tables[QL_ARCH.A8086][count], reg_data)
                        count += 1

                elif self.ql.archtype == QL_ARCH.X86:
                    for i in range(0, len(subcmd), 8):
                        reg_data = subcmd[i:i+7]
                        reg_data = int(reg_data, 16)
                        self.ql.reg.write(self.tables[QL_ARCH.X86][count], reg_data)
                        count += 1
                

                elif self.ql.archtype == QL_ARCH.X8664:
                    for i in range(0, 17*16, 16):
                        reg_data = subcmd[i:i+15]
                        reg_data = int(reg_data, 16)
                        self.ql.reg.write(self.tables[QL_ARCH.X8664][count], reg_data)
                        count += 1
                    for j in range(17*16, 17*16+15*8, 8):
                        reg_data = subcmd[j:j+7]
                        reg_data = int(reg_data, 16)
                        self.ql.reg.write(self.tables[QL_ARCH.X8664][count], reg_data)
                        count += 1
                
                elif self.ql.archtype == QL_ARCH.ARM:
                    for i in range(0, len(subcmd), 8):
                        reg_data = subcmd[i:i + 7]
                        reg_data = int(reg_data, 16)
                        self.ql.reg.write(self.tables[QL_ARCH.ARM][count], reg_data)
                        count += 1

                elif self.ql.archtype == QL_ARCH.ARM64:
                    for i in range(0, len(subcmd), 16):
                        reg_data = subcmd[i:i+15]
                        reg_data = int(reg_data, 16)
                        self.ql.reg.write(self.tables[QL_ARCH.ARM64][count], reg_data)
                        count += 1

                elif self.ql.archtype == QL_ARCH.MIPS:
                    for i in range(0, len(subcmd), 8):
                        reg_data = subcmd[i:i+7]
                        reg_data = int(reg_data, 16)
                        self.ql.reg.write(self.tables[QL_ARCH.MIPS][count], reg_data)
                        count += 1

                self.send('OK')


            def handle_H(subcmd):
                if subcmd.startswith('g'):
                    self.send('OK')
                if subcmd.startswith('c'):
                    self.send('OK')


            def handle_m(subcmd):
                addr, size = subcmd.split(',')
                addr = int(addr, 16)
                size = int(size, 16)

                try:
                    tmp = ''
                    for s in range(size):
                        mem = self.ql.mem.read(addr + s, 1)
                        mem = "".join(
                            [str("{:02x}".format(ord(c))) for c in mem.decode('latin1')])
                        tmp += mem
                    self.send(tmp)

                except:
                    self.send('E14')


            def handle_M(subcmd):
                addr, data = subcmd.split(',')
                size, data = data.split(':')
                addr = int(addr, 16)
                data = bytes.fromhex(data)
                try:
                    self.ql.mem.write(addr, data)
                    self.send('OK')
                except:
                    self.send('E01')


            def handle_p(subcmd):
                reg_index = int(subcmd, 16)
                reg_value = None
                try:
                    if self.ql.archtype== QL_ARCH.A8086:
                        if reg_index <= 9:
                            reg_value = self.ql.reg.read(self.tables[QL_ARCH.A8086][reg_index-1])
                        else:
                            reg_value = 0
                        reg_value = self.addr_to_str(reg_value)

                    elif self.ql.archtype== QL_ARCH.X86:
                        if reg_index <= 24:
                            reg_value = self.ql.reg.read(self.tables[QL_ARCH.X86][reg_index-1])
                        else:
                            reg_value = 0
                        reg_value = self.addr_to_str(reg_value)
                    
                    elif self.ql.archtype== QL_ARCH.X8664:
                        if reg_index <= 32:
                            reg_value = self.ql.reg.read(self.tables[QL_ARCH.X8664][reg_index-1])
                        else:
                            reg_value = 0
                        if reg_index <= 17:
                            reg_value = self.addr_to_str(reg_value)
                        elif 17 < reg_index:
                            reg_value = self.addr_to_str(reg_value, short = True)
                    
                    elif self.ql.archtype== QL_ARCH.ARM:
                        if reg_index < 17:
                            reg_value = self.ql.reg.read(self.tables[QL_ARCH.ARM][reg_index - 1])
                        else:
                            reg_value = 0
                        reg_value = self.addr_to_str(reg_value)

                    elif self.ql.archtype== QL_ARCH.ARM64:
                        if reg_index <= 32:
                            reg_value = self.ql.reg.read(self.tables[QL_ARCH.ARM64][reg_index - 1])
                        else:
                            reg_value = 0
                            reg_value = self.addr_to_str(reg_value)

                    elif self.ql.archtype== QL_ARCH.MIPS:
                        if reg_index <= 37:
                            reg_value = self.ql.reg.read(self.tables[QL_ARCH.MIPS][reg_index - 1])
                        else:
                            reg_value = 0
                        if self.ql.archendian == QL_ENDIAN.EL:
                            reg_value = self.addr_to_str(reg_value, endian="little")
                        else:
                            reg_value = self.addr_to_str(reg_value)
                    
                    if type(reg_value) is not str:
                        reg_value = self.addr_to_str(reg_value)

                    self.send(reg_value)
                except:
                    self.close()
                    raise


            def handle_P(subcmd):
                reg_index, reg_data = subcmd.split('=')
                reg_index = int(reg_index, 16)
                reg_name = self.tables[self.ql.archtype][reg_index]
                
                if self.ql.archtype== QL_ARCH.A8086:
                    reg_data = int(reg_data, 16)
                    reg_data = int.from_bytes(struct.pack('<I', reg_data), byteorder='big')
                    self.ql.reg.write(reg_name, reg_data)

                elif self.ql.archtype== QL_ARCH.X86:
                    reg_data = int(reg_data, 16)
                    reg_data = int.from_bytes(struct.pack('<I', reg_data), byteorder='big')
                    self.ql.reg.write(reg_name, reg_data)
                
                elif self.ql.archtype== QL_ARCH.X8664:
                    if reg_index <= 17:
                        reg_data = int(reg_data, 16)
                        reg_data = int.from_bytes(struct.pack('<Q', reg_data), byteorder='big')
                        self.ql.reg.write(reg_name, reg_data)
                    else:
                        reg_data = int(reg_data[:8], 16)
                        reg_data = int.from_bytes(struct.pack('<I', reg_data), byteorder='big')
                        self.ql.reg.write(reg_name, reg_data)
                
                elif self.ql.archtype== QL_ARCH.ARM:
                    reg_data = int(reg_data, 16)
                    reg_data = int.from_bytes(struct.pack('<I', reg_data), byteorder='big')
                    self.ql.reg.write(reg_name, reg_data)

                elif self.ql.archtype== QL_ARCH.ARM64:
                    reg_data = int(reg_data, 16)
                    reg_data = int.from_bytes(struct.pack('<Q', reg_data), byteorder='big')
                    self.ql.reg.write(reg_name, reg_data)

                elif self.ql.archtype== QL_ARCH.MIPS:
                    reg_data = int(reg_data, 16)
                    if self.ql.archendian == QL_ENDIAN.EL:
                        reg_data = int.from_bytes(struct.pack('<I', reg_data), byteorder='little')
                    else:
                        reg_data = int.from_bytes(struct.pack('<I', reg_data), byteorder='big')
                    self.ql.reg.write(reg_name, reg_data)

                if reg_name == self.ql.reg.arch_pc_name:
                    self.gdb.current_address = reg_data

                self.ql.log.info("gdb> Write to register %s with %x\n" % (self.tables[self.ql.archtype][reg_index], reg_data))
                self.send('OK')


            def handle_Q(subcmd):
                if subcmd.startswith('StartNoAckMode'):
                    self.send('OK')

                elif subcmd.startswith('DisableRandomization'):
                    self.send('OK')

                elif subcmd.startswith('ProgramSignals'):
                    self.send('OK')

                elif subcmd.startswith('NonStop'):
                    self.send('OK')

                elif subcmd.startswith('PassSignals'):
                    self.send('OK')

                elif subcmd.startswith('qemu'):
                    self.send('')

            def handle_D(subcmd):
                self.send('OK')

            def handle_q(subcmd):
                if subcmd.startswith('Supported:'):
                    # might or might not need for multi thread
                    if self.ql.multithread == False:
                        self.send("PacketSize=3fff;QPassSignals+;QProgramSignals+;QStartupWithShell+;QEnvironmentHexEncoded+;QEnvironmentReset+;QEnvironmentUnset+;QSetWorkingDir+;QCatchSyscalls+;qXfer:libraries-svr4:read+;augmented-libraries-svr4-read+;qXfer:auxv:read+;qXfer:spu:read+;qXfer:spu:write+;qXfer:siginfo:read+;qXfer:siginfo:write+;qXfer:features:read+;QStartNoAckMode+;qXfer:osdata:read+;multiprocess+;fork-events+;vfork-events+;exec-events+;QNonStop+;QDisableRandomization+;qXfer:threads:read+;ConditionalTracepoints+;TraceStateVariables+;TracepointSource+;DisconnectedTracing+;StaticTracepoints+;InstallInTrace+;qXfer:statictrace:read+;qXfer:traceframe-info:read+;EnableDisableTracepoints+;QTBuffer:size+;tracenz+;ConditionalBreakpoints+;BreakpointCommands+;QAgent+;swbreak+;hwbreak+;qXfer:exec-file:read+;no-resumed+")
                    else:    
                        self.send("PacketSize=47ff;QPassSignals+;QProgramSignals+;QStartupWithShell+;QEnvironmentHexEncoded+;QEnvironmentReset+;QEnvironmentUnset+;QSetWorkingDir+;QCatchSyscalls+;qXfer:libraries-svr4:read+;augmented-libraries-svr4-read+;qXfer:auxv:read+;qXfer:siginfo:read+;qXfer:siginfo:write+;qXfer:features:read+;QStartNoAckMode+;qXfer:osdata:read+;multiprocess+;fork-events+;vfork-events+;exec-events+;QNonStop+;QDisableRandomization+;qXfer:threads:read+;ConditionalTracepoints+;TraceStateVariables+;TracepointSource+;DisconnectedTracing+;FastTracepoints+;StaticTracepoints+;InstallInTrace+;qXfer:statictrace:read+;qXfer:traceframe-info:read+;EnableDisableTracepoints+;QTBuffer:size+;tracenz+;ConditionalBreakpoints+;BreakpointCommands+;QAgent+;Qbtrace:bts+;Qbtrace-conf:bts:size+;Qbtrace:pt+;Qbtrace-conf:pt:size+;Qbtrace:off+;qXfer:btrace:read+;qXfer:btrace-conf:read+;swbreak+;hwbreak+;qXfer:exec-file:read+;vContSupported+;QThreadEvents+;no-resumed+")
                elif subcmd.startswith('Xfer:features:read'):
                    xfercmd_file    = subcmd.split(':')[3]
                    xfercmd_abspath = os.path.dirname(os.path.abspath(__file__))
                    xml_folder      = arch_convert_str(self.ql.archtype)
                    xfercmd_file    = os.path.join(xfercmd_abspath,"xml",xml_folder, xfercmd_file)                        

                    if os.path.exists(xfercmd_file) and self.ql.ostype is not QL_OS.WINDOWS:
                        with open(xfercmd_file, 'r') as f:
                            file_contents = f.read()
                            self.send("l%s" % file_contents)
                    else:
                        self.ql.log.info("gdb> Platform is not supported by xml or xml file not found: %s\n" % (xfercmd_file))
                        self.send("l")


                elif subcmd.startswith('Xfer:threads:read::0,'):
                    if self.ql.ostype in QL_OS_NONPID:
                        self.send("l")
                    else:    
                        file_contents = ("<threads>\r\n<thread id=\""+ str(self.ql.os.pid) + "\" core=\"1\" name=\"" + self.ql.targetname + "\"/>\r\n</threads>")
                        self.send("l" + file_contents)

                elif subcmd.startswith('Xfer:auxv:read::'):
                    if self.ql.code:
                        return
                    if self.ql.ostype in (QL_OS.LINUX, QL_OS.FREEBSD) :
                        if self.ql.archbit == 64:
                            ANNEX               = "00000000000000"
                            AT_SYSINFO_EHDR     = "0000000000000000" # System-supplied DSO's ELF header
                            ID_AT_HWCAP         = "1000000000000000"
                            ID_AT_PAGESZ        = "0600000000000000"
                            ID_AT_CLKTCK        = "1100000000000000"
                            AT_CLKTCK           = "6400000000000000" # Frequency of times() 100
                            ID_AT_PHDR          = "0300000000000000"
                            ID_AT_PHENT         = "0400000000000000"
                            ID_AT_PHNUM         = "0500000000000000"
                            ID_AT_BASE          = "0700000000000000"
                            ID_AT_FLAGS         = "0800000000000000"
                            ID_AT_ENTRY         = "0900000000000000"
                            ID_AT_UID           = "0b00000000000000"
                            ID_AT_EUID          = "0c00000000000000"
                            ID_AT_GID           = "0d00000000000000"
                            ID_AT_EGID          = "0e00000000000000"
                            ID_AT_SECURE        = "1700000000000000"
                            AT_SECURE           = "0000000000000000"
                            ID_AT_RANDOM        = "1900000000000000"
                            ID_AT_HWCAP2        = "1a00000000000000"
                            AT_HWCAP2           = "0000000000000000"
                            ID_AT_EXECFN        = "1f00000000000000"
                            AT_EXECFN           = "0000000000000000" # File name of executable
                            ID_AT_PLATFORM      = "0f00000000000000"
                            ID_AT_NULL          = "0000000000000000"
                            AT_NULL             = "0000000000000000"

                        elif self.ql.archbit == 32:
                            ANNEX           = "000000"
                            AT_SYSINFO_EHDR = "00000000"  # System-supplied DSO's ELF header
                            ID_AT_HWCAP     = "10000000"
                            ID_AT_PAGESZ    = "06000000"
                            ID_AT_CLKTCK    = "11000000"
                            AT_CLKTCK       = "64000000"  # Frequency of times() 100
                            ID_AT_PHDR      = "03000000"
                            ID_AT_PHENT     = "04000000"
                            ID_AT_PHNUM     = "05000000"
                            ID_AT_BASE      = "07000000"
                            ID_AT_FLAGS     = "08000000"
                            ID_AT_ENTRY     = "09000000"
                            ID_AT_UID       = "0b000000"
                            ID_AT_EUID      = "0c000000"
                            ID_AT_GID       = "0d000000"
                            ID_AT_EGID      = "0e000000"
                            ID_AT_SECURE    = "17000000"
                            AT_SECURE       = "00000000"
                            ID_AT_RANDOM    = "19000000"
                            ID_AT_HWCAP2    = "1a000000"
                            AT_HWCAP2       = "00000000"
                            ID_AT_EXECFN    = "1f000000"
                            AT_EXECFN       = "00000000"  # File name of executable
                            ID_AT_PLATFORM  = "0f000000"
                            ID_AT_NULL      = "00000000"
                            AT_NULL         = "00000000"

                        AT_HWCAP    = self.addr_to_str(self.ql.loader.elf_hwcap)  # mock cpuid 0x1f8bfbff
                        AT_PAGESZ   = self.addr_to_str(self.ql.loader.elf_pagesz)  # System page size, fixed in qiling
                        AT_PHDR     = self.addr_to_str(self.ql.loader.elf_phdr)  # Program headers for program
                        AT_PHENT    = self.addr_to_str(self.ql.loader.elf_phent)  # Size of program header entry
                        AT_PHNUM    = self.addr_to_str(self.ql.loader.elf_phnum)  # Number of program headers
                        AT_BASE     = self.addr_to_str(self.ql.loader.interp_address)  # Base address of interpreter
                        AT_FLAGS    = self.addr_to_str(self.ql.loader.elf_flags)
                        AT_ENTRY    = self.addr_to_str(self.ql.loader.elf_entry)  # Entry point of program
                        AT_UID      = self.addr_to_str(self.ql.loader.elf_guid)  # UID from ql.profile
                        AT_EUID     = self.addr_to_str(self.ql.loader.elf_guid)  # UID from ql.profile
                        AT_GID      = self.addr_to_str(self.ql.loader.elf_guid)  # UID from ql.profile
                        AT_EGID     = self.addr_to_str(self.ql.loader.elf_guid)  # UID from ql.profile
                        AT_RANDOM   = self.addr_to_str(self.ql.loader.randstraddr)  # Address of 16 random bytes
                        AT_PLATFORM = self.addr_to_str(self.ql.loader.cpustraddr)  # String identifying platform

                        auxvdata_c = (
                                        ANNEX + AT_SYSINFO_EHDR +
                                        ID_AT_HWCAP + AT_HWCAP +
                                        ID_AT_PAGESZ + AT_PAGESZ +
                                        ID_AT_CLKTCK + AT_CLKTCK +
                                        ID_AT_PHDR + AT_PHDR +
                                        ID_AT_PHENT + AT_PHENT +
                                        ID_AT_PHNUM + AT_PHNUM +
                                        ID_AT_BASE + AT_BASE +
                                        ID_AT_FLAGS + AT_FLAGS +
                                        ID_AT_ENTRY + AT_ENTRY +
                                        ID_AT_UID + AT_UID +
                                        ID_AT_EUID + AT_EUID +
                                        ID_AT_GID + AT_GID +
                                        ID_AT_EGID + AT_EGID +
                                        ID_AT_SECURE + AT_SECURE +
                                        ID_AT_RANDOM + AT_RANDOM +
                                        ID_AT_HWCAP2 + AT_HWCAP2 +
                                        ID_AT_EXECFN + AT_EXECFN +
                                        ID_AT_PLATFORM + AT_PLATFORM +
                                        ID_AT_NULL + AT_NULL
                                    )

                        auxvdata = self.bin_to_escstr(unhexlify(auxvdata_c))
                        #self.send(b'l!%s' % auxvdata)
                    else:
                        auxvdata = b""
                    
                    self.send(b'l!%s' % auxvdata)

                elif subcmd.startswith('Xfer:exec-file:read:'):
                    self.send("l%s" % str(self.exe_abspath))


                elif subcmd.startswith('Xfer:libraries-svr4:read:'):
                    if self.ql.ostype in (QL_OS.LINUX, QL_OS.FREEBSD):
                        xml_addr_mapping=("<library-list-svr4 version=\"1.0\">")
                        """
                        FIXME: need to find out when do we need this
                        """
                        #for s, e, info in self.ql.map_info:
                        #    addr_mapping += ("<library name=\"%s\" lm=\"0x%x\" l_addr=\"%x\" l_ld=\"\"/>" %(info, e, s)) 
                        xml_addr_mapping += ("</library-list-svr4>")
                        self.send("l%s" % xml_addr_mapping)
                    else:     
                        self.send("l<library-list-svr4 version=\"1.0\"></library-list-svr4>")

                elif subcmd.startswith("Xfer:btrace-conf:read:"):
                     self.send("E.Btrace not enabled.")

                elif subcmd == "Attached":
                    self.send("")

                elif subcmd.startswith("C"):
                    self.send("")

                elif subcmd.startswith("L:"):
                    self.send("M001")

                elif subcmd == "fThreadInfo":
                    self.send("m0")

                elif subcmd == "sThreadInfo":
                    self.send("l")

                elif subcmd == ("TStatus"):
                    self.send("T0;tnotrun:0;tframes:0;tcreated:0;tfree:50*!;tsize:50*!;circular:0;disconn:0;starttime:0;stoptime:0;username:;notes::")

                elif subcmd == ("TfV"):
                    self.send("l")

                elif subcmd == ("TsV"):
                    self.send("l")

                elif subcmd == ("TfP"):
                    self.send("l")

                elif subcmd == ("TsP"):
                    self.send("l")


                elif subcmd.startswith("Symbol"):
                    self.send("")

                elif subcmd.startswith("Attached"):
                    self.send("")

                elif subcmd == "Offsets":
                    self.send("Text=0;Data=0;Bss=0")


            def handle_v(subcmd):

                if subcmd == 'MustReplyEmpty':
                    self.send("")

                elif subcmd.startswith('File:open'):
                    (file_path, flags, mode) = subcmd.split(':')[-1].split(',')
                    file_path = unhexlify(file_path).decode(encoding='UTF-8')
                    flags = int(flags, base=16)
                    mode = int(mode, base=16)
                    if file_path.startswith(self.rootfs_abspath):
                        file_abspath = file_path
                    else:
                        file_abspath = self.ql.os.transform_to_real_path(file_path)
                    
                    self.ql.log.debug("gdb> target file: %s" % (file_abspath))
                    if os.path.exists(file_abspath) and not (file_path).startswith("/proc"):
                        fd = os.open(file_abspath, flags, mode)
                        self.send("F%x" % fd)
                    else:
                        self.send("F-1")

                elif subcmd.startswith('File:pread:'):
                    (fd, count, offset) = subcmd.split(':')[-1].split(',')

                    fd = int(fd, base=16)
                    offset = int(offset, base=16)
                    count = int(count, base=16)

                    data = os.pread(fd, count, offset)
                    size = len(data)
                    data = self.bin_to_escstr(data)

                    if data:
                        self.send(("F%x;" % size).encode() + (data))
                    else:
                        self.send("F0;")

                elif subcmd.startswith('File:close'):
                    fd = subcmd.split(':')[-1]
                    fd = int(fd, base=16)
                    os.close(fd)
                    self.send("F0")

                elif subcmd.startswith('Kill'):
                    self.send('OK')

                elif subcmd.startswith('Cont'):
                    self.ql.log.debug("gdb> Cont command received: %s" % subcmd)
                    if subcmd == 'Cont?':
                        self.send('vCont;c;C;t;s;S;r')
                    elif subcmd.startswith ("Cont;"):
                        subcmd = subcmd.split(';')
                        subcmd = subcmd[1].split(':')
                        if subcmd[0] in ('c', 'C05'):
                            handle_c(subcmd)
                        elif subcmd[0] in ('S', 's', 'S05'):
                            handle_s(subcmd)
                else:
                    self.send("")


            def handle_s(subcmd):
                current_address = self.gdb.current_address
                if current_address is None:
                    entry_point = self.gdb.entry_point
                    if entry_point is not None:
                        self.gdb.soft_bp = True
                        self.gdb.resume_emu(entry_point)
                else:
                    self.gdb.soft_bp = True
                    self.gdb.resume_emu()
                self.send('S%.2x' % GDB_SIGNAL_TRAP)


            def handle_X(subcmd):
                self.send('')


            def handle_Z(subcmd):
                data = subcmd
                ztype = data[data.find('Z') + 1:data.find(',')]
                if ztype == '0':
                    ztype, address, value = data.split(',')
                    address = int(address, 16)
                    try:
                        self.gdb.bp_insert(address)
                        self.send('OK')
                    except:
                        self.send('E22')
                else:
                    self.send('E22')


            def handle_z(subcmd):
                data = subcmd.split(',')
                if len(data) != 3:
                    self.send('E22')
                try:
                    type = data[0]
                    addr = int(data[1], 16)
                    length = data[2]
                    self.gdb.bp_remove(addr, type, length)
                    self.send('OK')
                except:
                    self.send('E22')


            def handle_exclaim(subcmd):
                self.send('OK')

            commands = {
                '!': handle_exclaim,
                '?': handle_qmark,
                'c': handle_c,
                'C': handle_C,
                'D': handle_D,
                'g': handle_g,
                'G': handle_G,
                'H': handle_H,
                'm': handle_m,
                'M': handle_M,
                'p': handle_p,
                'P': handle_P,
                'q': handle_q,
                'Q': handle_Q,
                's': handle_s,
                'v': handle_v,
                'X': handle_X,
                'Z': handle_Z,
                'z': handle_z
            }

            cmd, subcmd = pkt[0], pkt[1:]
            if cmd == 'k':
                break

            if cmd not in commands:
                self.send('')
                self.ql.log.info("gdb> Command not supported: %s\n" %(cmd))
                continue
            self.ql.log.debug("gdb> received: %s%s" % (cmd, subcmd))
            commands[cmd](subcmd)

        self.close()


    def receive(self):
        '''Receive a packet from a GDB client'''
        csum = 0
        state = 'Finding SOP'
        packet = ''
        try:
            while True:
                c = self.netin.read(1)
                if c == '\x03':
                    return 'Error: CTRL+C'

                if len(c) != 1:
                    return 'Error: EOF'

                if state == 'Finding SOP':
                    if c == '$':
                        state = 'Finding EOP'
                elif state == 'Finding EOP':
                    if c == '#':
                        if csum != int(self.netin.read(2), 16):
                            raise Exception('invalid checksum')
                        self.last_pkt = packet
                        return 'Good'
                    else:
                        packet += c
                        csum = (csum + ord(c)) & 0xff
                else:
                    raise Exception('should not be here')
        except:
            self.close()
            raise

    def checksum(self, data):
        checksum = 0
        for c in data:
            if type(c) == str:
                checksum += (ord(c))
            else:
                checksum += c
        return checksum & 0xff

    def send(self, msg):
        """Send a packet to the GDB client"""
        if type(msg) == str:
            self.send_raw('$%s#%.2x' % (msg, self.checksum(msg)))
        else:
            self.clientsocket.send(b'$%s#%.2x' % (msg, self.checksum(msg)))
            self.netout.flush()

        self.ql.log.debug("gdb> send: $%s#%.2x" % (msg, self.checksum(msg)))

    def send_raw(self, r):
        self.netout.write(r)
        self.netout.flush()



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
from qiling.os.windows.const import *
from qiling.os.windows.fncc import *
from qiling.os.const import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *

@winsdkapi(cc=STDCALL)
def hook_WudfIsUserDebuggerPresent(ql, address, params):
    return 0


@winsdkapi(cc=STDCALL)
def hook_WudfIsAnyDebuggerPresent(ql, address, params):
    return 0


@winsdkapi(cc=STDCALL)
def hook_WudfIsKernelDebuggerPresent(ql, address, params):
    return 0



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import binascii

from qiling.const import *
from qiling.os.const import *

from qiling.os.uefi.const import EFI_SUCCESS, EFI_NOT_FOUND, EFI_OUT_OF_RESOURCES
from qiling.os.uefi.utils import *
from qiling.os.uefi.fncc import *
from qiling.os.uefi.ProcessorBind import *
from qiling.os.uefi.UefiBaseType import *
from qiling.os.uefi.UefiMultiPhase import *
from qiling.os.uefi.UefiSpec import *
from qiling.os.uefi.protocols import common
from qiling.os.uefi import rt

# @see: MdePkg\Include\Pi\PiSmmCis.h

class EFI_MM_IO_WIDTH(ENUM):
	_members_ = [
		'MM_IO_UINT8',
		'MM_IO_UINT16',
		'MM_IO_UINT32',
		'MM_IO_UINT64'
	]

EFI_MM_HANDLER_ENTRY_POINT = FUNCPTR(EFI_STATUS, EFI_HANDLE, PTR(VOID), PTR(VOID), PTR(UINTN))
EFI_MM_NOTIFY_FN = FUNCPTR(EFI_STATUS, PTR(EFI_GUID), PTR(VOID), EFI_HANDLE)

class EFI_MM_IO_ACCESS(STRUCT):
	EFI_SMM_CPU_IO2_PROTOCOL = STRUCT

	_fields_ = [
		('Read',	FUNCPTR(EFI_STATUS, PTR(EFI_SMM_CPU_IO2_PROTOCOL), EFI_MM_IO_WIDTH, UINT64, UINTN, PTR(VOID))),
		('Write',	FUNCPTR(EFI_STATUS, PTR(EFI_SMM_CPU_IO2_PROTOCOL), EFI_MM_IO_WIDTH, UINT64, UINTN, PTR(VOID))),
	]

class EFI_SMM_CPU_IO2_PROTOCOL(STRUCT):
	_fields_ = [
		('Mem',	EFI_MM_IO_ACCESS),
		('Io',	EFI_MM_IO_ACCESS)
	]

class EFI_SMM_SYSTEM_TABLE2(STRUCT):
	EFI_SMM_SYSTEM_TABLE2 = STRUCT

	_fields_ = [
		('Hdr',								EFI_TABLE_HEADER),
		('SmmFirmwareVendor',				PTR(CHAR16)),
		('SmmFirmwareRevision',				UINT32),
		('PADDING_0',						UINT8 * 4),
		('SmmInstallConfigurationTable',	FUNCPTR(EFI_STATUS, PTR(EFI_SMM_SYSTEM_TABLE2), PTR(EFI_GUID), PTR(VOID), UINTN)),
		('SmmIo',							EFI_SMM_CPU_IO2_PROTOCOL),
		('SmmAllocatePool',					FUNCPTR(EFI_STATUS, EFI_MEMORY_TYPE, UINTN, PTR(PTR(VOID)))),
		('SmmFreePool',						FUNCPTR(EFI_STATUS, PTR(VOID))),
		('SmmAllocatePages',				FUNCPTR(EFI_STATUS, EFI_ALLOCATE_TYPE, EFI_MEMORY_TYPE, UINTN, PTR(EFI_PHYSICAL_ADDRESS))),
		('SmmFreePages',					FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, UINTN)),
		('SmmStartupThisAp',				FUNCPTR(EFI_STATUS, FUNCPTR(VOID, PTR(VOID)), UINTN, PTR(VOID))),
		('CurrentlyExecutingCpu',			UINTN),
		('NumberOfCpus',					UINTN),
		('CpuSaveStateSize',				PTR(UINTN)),
		('CpuSaveState',					PTR(PTR(VOID))),
		('NumberOfTableEntries',			UINTN),
		('SmmConfigurationTable',			PTR(EFI_CONFIGURATION_TABLE)),
		('SmmInstallProtocolInterface',		FUNCPTR(EFI_STATUS, PTR(EFI_HANDLE), PTR(EFI_GUID), EFI_INTERFACE_TYPE, PTR(VOID))),
		('SmmUninstallProtocolInterface',	FUNCPTR(EFI_STATUS, PTR(VOID), PTR(EFI_GUID), PTR(VOID))),
		('SmmHandleProtocol',				FUNCPTR(EFI_STATUS, PTR(VOID), PTR(EFI_GUID), PTR(PTR(VOID)))),
		('SmmRegisterProtocolNotify',		FUNCPTR(EFI_STATUS, PTR(EFI_GUID), EFI_MM_NOTIFY_FN, PTR(PTR(VOID)))),
		('SmmLocateHandle',					FUNCPTR(EFI_STATUS, EFI_LOCATE_SEARCH_TYPE, PTR(EFI_GUID), PTR(VOID), PTR(UINTN), PTR(EFI_HANDLE))),
		('SmmLocateProtocol',				FUNCPTR(EFI_STATUS, PTR(EFI_GUID), PTR(VOID), PTR(PTR(VOID)))),
		('SmiManage',						FUNCPTR(EFI_STATUS, PTR(EFI_GUID), PTR(VOID), PTR(VOID), PTR(UINTN))),
		('SmiHandlerRegister',				FUNCPTR(EFI_STATUS, EFI_MM_HANDLER_ENTRY_POINT, PTR(EFI_GUID), PTR(EFI_HANDLE))),
		('SmiHandlerUnRegister',			FUNCPTR(EFI_STATUS, EFI_HANDLE)),
	]

@dxeapi(params = {
	"Guid"	: GUID,		# PTR(EFI_GUID)
	"Table"	: POINTER	# PTR(VOID)
})
def hook_SmmInstallConfigurationTable(ql, address, params):
	return common.InstallConfigurationTable(ql.loader.smm_context, params)

@dxeapi(params = {
	"type"		: INT,			# EFI_ALLOCATE_TYPE
	"MemoryType": INT,			# EFI_MEMORY_TYPE
	"Pages"		: ULONGLONG,	# UINTN
	"Memory"	: POINTER		# PTR(EFI_PHYSICAL_ADDRESS)
})
def hook_SmmAllocatePages(ql, address, params):
	alloc_size = params["Pages"] * PAGE_SIZE

	if params['type'] == EFI_ALLOCATE_TYPE.AllocateAddress:
		address = read_int64(ql, params["Memory"])

		# TODO: check the range [address, address + alloc_size] is available first
		ql.mem.map(address, alloc_size)
	else:
		# TODO: allocate memory according to 'MemoryType'
		address = ql.loader.smm_context.heap.alloc(alloc_size)

		if address == 0:
			return EFI_OUT_OF_RESOURCES

		write_int64(ql, params["Memory"], address)

	return EFI_SUCCESS

@dxeapi(params = {
	"Memory"	: ULONGLONG,	# EFI_PHYSICAL_ADDRESS
	"Pages"		: ULONGLONG		# UINTN
})
def hook_SmmFreePages(ql, address, params):
	address = params["Memory"]
	alloc_size = params["Pages"] * PAGE_SIZE

	ret = ql.mem.free(address, alloc_size)

	return EFI_SUCCESS if ret else EFI_INVALID_PARAMETER

@dxeapi(params = {
	"PoolType"	: INT,		# EFI_MEMORY_TYPE
	"Size"		: INT,		# UINTN
	"Buffer"	: POINTER	# PTR(PTR(VOID))
})
def hook_SmmAllocatePool(ql, address, params):
	# TODO: allocate memory acording to "PoolType"
	address = ql.loader.smm_context.heap.alloc(params["Size"])
	write_int64(ql, params["Buffer"], address)

	return EFI_SUCCESS if address else EFI_OUT_OF_RESOURCES

@dxeapi(params = {
	"Buffer": POINTER # PTR(VOID)
})
def hook_SmmFreePool(ql, address, params):
	address = params["Buffer"]
	ret = ql.loader.smm_context.heap.free(address)

	return EFI_SUCCESS if ret else EFI_INVALID_PARAMETER

@dxeapi(params = {
	"Procedure"		: POINTER,
	"CpuNumber"		: INT,
	"ProcArguments"	: POINTER
})
def hook_SmmStartupThisAp(ql, address, params):
	return EFI_INVALID_PARAMETER

@dxeapi(params = {
	"Handle"		: POINTER,		# PTR(EFI_HANDLE)
	"Protocol"		: GUID,			# PTR(EFI_GUID)
	"InterfaceType"	: ULONGLONG,	# EFI_INTERFACE_TYPE
	"Interface"		: POINTER,		# PTR(VOID)
})
def hook_SmmInstallProtocolInterface(ql, address, params):
	return common.InstallProtocolInterface(ql.loader.smm_context, params)

@dxeapi(params = {
	"Handle"	: POINTER,	# EFI_HANDLE
	"Protocol"	: GUID,		# PTR(EFI_GUID)
	"Interface"	: POINTER	# PTR(VOID)
})
def hook_SmmUninstallProtocolInterface(ql, address, params):
	return common.UninstallProtocolInterface(ql.loader.smm_context, params)

@dxeapi(params = {
	"Handle"	: POINTER,	# EFI_HANDLE
	"Protocol"	: GUID,		# PTR(EFI_GUID)
	"Interface"	: POINTER	# PTR(PTR(VOID))
})
def hook_SmmHandleProtocol(ql, address, params):
	return common.HandleProtocol(ql.loader.smm_context, params)

@dxeapi(params = {
	"Protocol"		: GUID,		# PTR(EFI_GUID)
	"Function"		: POINTER,	# EFI_MM_NOTIFY_FN
	"Registration"	: POINTER	# PTR(PTR(VOID))
})
def hook_SmmRegisterProtocolNotify(ql, address, params):
	#event = params['Event']
	#proto = params["Protocol"]
	#
	#if event in ql.loader.events:
	#	ql.loader.events[event]['Guid'] = proto
	#	check_and_notify_protocols(ql)
	#
	#	return EFI_SUCCESS

	return EFI_INVALID_PARAMETER

@dxeapi(params = {
	"SearchType": INT,		# EFI_LOCATE_SEARCH_TYPE
	"Protocol"	: GUID,		# PTR(EFI_GUID)
	"SearchKey"	: POINTER,	# PTR(VOID)
	"BufferSize": POINTER,	# PTR(UINTN)
	"Buffer"	: POINTER	# PTR(EFI_HANDLE)
})
def hook_SmmLocateHandle(ql, address, params):
	return common.LocateHandle(ql.loader.smm_context, params)

@dxeapi(params = {
	"Protocol"		: GUID,		# PTR(EFI_GUID)
	"Registration"	: POINTER,	# PTR(VOID)
	"Interface"		: POINTER	# PTR(PTR(VOID))
})
def hook_SmmLocateProtocol(ql, address, params):
	return common.LocateProtocol(ql.loader.smm_context, params)

@dxeapi(params = {
	"HandlerType"	: GUID,
	"Context"		: POINTER,
	"CommBuffer"	: POINTER,
	"CommBufferSize": POINTER
})
def hook_SmiManage(ql, address, params):
	return EFI_NOT_FOUND

@dxeapi(params = {
	"Handler"		: POINTER,
	"HandlerType"	: GUID,
	"DispatchHandle": POINTER
})
def hook_SmiHandlerRegister(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params = {
	"DispatchHandle": POINTER
})
def hook_SmiHandlerUnRegister(ql, address, params):
	return EFI_SUCCESS

def initialize(ql, gSmst : int):
	ql.loader.gSmst = gSmst

	gSmmRT = gSmst + EFI_SMM_SYSTEM_TABLE2.sizeof()	# smm runtime services
	cfg = gSmmRT + EFI_RUNTIME_SERVICES.sizeof()	# configuration tables array

	rt.initialize(ql, gSmmRT)

	descriptor = {
		'struct' : EFI_SMM_SYSTEM_TABLE2,
		'fields' : (
			('Hdr',								None),
			('SmmFirmwareVendor',				None),
			('SmmFirmwareRevision',				None),
			('PADDING_0',						None),
			('SmmInstallConfigurationTable',	hook_SmmInstallConfigurationTable),
			('SmmIo',							None),
			('SmmAllocatePool',					hook_SmmAllocatePool),
			('SmmFreePool',						hook_SmmFreePool),
			('SmmAllocatePages',				hook_SmmAllocatePages),
			('SmmFreePages',					hook_SmmFreePages),
			('SmmStartupThisAp',				hook_SmmStartupThisAp),
			('CurrentlyExecutingCpu',			None),
			('NumberOfCpus',					None),
			('CpuSaveStateSize',				None),
			('CpuSaveState',					None),
			('NumberOfTableEntries',			0),
			('SmmConfigurationTable',			cfg),
			('SmmInstallProtocolInterface',		hook_SmmInstallProtocolInterface),
			('SmmUninstallProtocolInterface',	hook_SmmUninstallProtocolInterface),
			('SmmHandleProtocol',				hook_SmmHandleProtocol),
			('SmmRegisterProtocolNotify',		hook_SmmRegisterProtocolNotify),
			('SmmLocateHandle',					hook_SmmLocateHandle),
			('SmmLocateProtocol',				hook_SmmLocateProtocol),
			('SmiManage',						hook_SmiManage),
			('SmiHandlerRegister',				hook_SmiHandlerRegister),
			('SmiHandlerUnRegister',			hook_SmiHandlerUnRegister),
		)
	}

	instance = init_struct(ql, gSmst, descriptor)
	instance.saveTo(ql, gSmst)

	# configuration tables bookkeeping
	confs = []

	# these are needed for utils.SmmInstallConfigurationTable
	ql.loader.smm_context.conf_table_array = confs
	ql.loader.smm_context.conf_table_array_ptr = cfg

	# configuration table data space; its location is calculated by leaving
	# enough space for 100 configuration table entries. only a few entries are
	# expected, so 100 should definitely suffice
	conf_data = cfg + EFI_CONFIGURATION_TABLE.sizeof() * 100
	ql.loader.smm_context.conf_table_data_ptr = conf_data
	ql.loader.smm_context.conf_table_data_next_ptr = conf_data

	install_configuration_table(ql.loader.smm_context, "SMM_RUNTIME_SERVICES_TABLE", gSmmRT)
	


__all__ = [
	'EFI_SMM_SYSTEM_TABLE2',
	'initialize'
]



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
from qiling.os.windows.const import *
from qiling.os.windows.fncc import *
from qiling.os.const import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
from qiling.os.windows.const import *
from qiling.os.windows.fncc import *
from qiling.os.const import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *

dllname = 'msi_dll'

# UINT MsiGetComponentStateA(
#   MSIHANDLE    hInstall,
#   LPCSTR       szComponent,
#   INSTALLSTATE *piInstalled,
#   INSTALLSTATE *piAction
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_MsiGetComponentStateA(ql, address, params):
    return 6  # INVALID_HANDLE



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from ctypes import Structure
from ctypes import c_uint32, c_uint16

from .base import QlBaseCoverage


# Adapted from https://www.ayrx.me/drcov-file-format
class bb_entry(Structure):
    _fields_ = [
        ("start",  c_uint32),
        ("size",   c_uint16),
        ("mod_id", c_uint16)
    ]

class QlDrCoverage(QlBaseCoverage):
    """
    Collects emulated code coverage and formats it in accordance with the DynamoRIO based
    tool drcov: https://dynamorio.org/dynamorio_docs/page_drcov.html

    The resulting output file can later be imported by coverage visualization tools such
    as Lighthouse: https://github.com/gaasedelen/lighthouse
    """

    FORMAT_NAME = "drcov"

    def __init__(self, ql):
        super().__init__()
        self.ql            = ql
        self.drcov_version = 2
        self.drcov_flavor  = 'drcov'
        self.basic_blocks  = []
        self.bb_callback   = None

    @staticmethod
    def block_callback(ql, address, size, self):
        for mod_id, mod in enumerate(ql.loader.images):
            if mod.base <= address <= mod.end:
                ent = bb_entry(address - mod.base, size, mod_id)
                self.basic_blocks.append(ent)
                break

    def activate(self):
        self.bb_callback = self.ql.hook_block(self.block_callback, user_data=self)

    def deactivate(self):
        self.ql.hook_del(self.bb_callback)

    def dump_coverage(self, coverage_file):
        with open(coverage_file, "wb") as cov:
            cov.write(f"DRCOV VERSION: {self.drcov_version}\n".encode())
            cov.write(f"DRCOV FLAVOR: {self.drcov_flavor}\n".encode())
            cov.write(f"Module Table: version {self.drcov_version}, count {len(self.ql.loader.images)}\n".encode())
            cov.write("Columns: id, base, end, entry, checksum, timestamp, path\n".encode())
            for mod_id, mod in enumerate(self. ql.loader.images):
                cov.write(f"{mod_id}, {mod.base}, {mod.end}, 0, 0, 0, {mod.path}\n".encode())
            cov.write(f"BB Table: {len(self.basic_blocks)} bbs\n".encode())
            for bb in self.basic_blocks:
                cov.write(bytes(bb))



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

IDCANCEL = 2

IDI_APPLICATION = 32512
IDI_ASTERISK = 32516
IDI_ERROR = 32513
IDI_EXCLAMATION = 32515
IDI_HAND = 32513
IDI_INFORMATION = 32516
IDI_QUESTION = 32514
IDI_SHIELD = 32518
IDI_WARNING = 32515
IDI_WINLOGO = 32517


#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.os.windows.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.thread import *
from qiling.exception import *
import configparser
from qiling.os.windows.structs import *


dllname = 'kernel32_dll'

# __analysis_noreturn VOID FatalExit(
#   int ExitCode
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_FatalExit(ql, address, params):
    ql.emu_stop()
    ql.os.PE_RUN = False


# PVOID EncodePointer(
#  _In_ PVOID Ptr
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={"Ptr": POINTER})
def hook_EncodePointer(ql, address, params):
    return params['Ptr']


# PVOID DecodePointer(
#  _In_ PVOID Ptr
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={"Ptr": POINTER})
def hook_DecodePointer(ql, address, params):
    return params['Ptr']


# UINT WinExec(
#   LPCSTR lpCmdLine,
#   UINT   uCmdShow
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_WinExec(ql, address, params):
    return 33


# DECLSPEC_ALLOCATOR HLOCAL LocalAlloc(
#   UINT   uFlags,
#   SIZE_T uBytes
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LocalAlloc(ql, address, params):
    ret = ql.os.heap.alloc(params["uBytes"])
    return ret


# DECLSPEC_ALLOCATOR HLOCAL LocalReAlloc(
#   _Frees_ptr_opt_ HLOCAL hMem,
#   SIZE_T                 uBytes,
#   UINT                   uFlags
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LocalReAlloc(ql, address, params):
    old_mem = params["hMem"]
    ql.os.heap.free(old_mem)
    ret = ql.os.heap.alloc(params["uBytes"])
    return ret


# HLOCAL LocalFree(
#   _Frees_ptr_opt_ HLOCAL hMem
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LocalFree(ql, address, params):
    old_mem = params["hMem"]
    ql.os.heap.free(old_mem)
    return 0


# UINT SetHandleCount(
#   UINT uNumber
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_SetHandleCount(ql, address, params):
    uNumber = params["uNumber"]
    return uNumber


# LPVOID GlobalLock(
#  HGLOBAL hMem
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GlobalLock(ql, address, params):
    return params['hMem']


# LPVOID GlobalUnlock(
#  HGLOBAL hMem
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GlobalUnlock(ql, address, params):
    return 1


# DECLSPEC_ALLOCATOR HGLOBAL GlobalAlloc(
#  UINT   uFlags,
#  SIZE_T dwBytes
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'SIZE_T': 'UINT'})
def hook_GlobalAlloc(ql, address, params):
    return ql.os.heap.alloc(params["dwBytes"])


# HGLOBAL GlobalFree(
#   _Frees_ptr_opt_ HGLOBAL hMem
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GlobalFree(ql, address, params):
    old_mem = params["hMem"]
    ql.os.heap.free(old_mem)
    return 0


# HGLOBAL GlobalHandle(
#   LPCVOID pMem
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GlobalHandle(ql, address, params):
    return params["pMem"]


# LPSTR lstrcpynA(
#   LPSTR  lpString1,
#   LPCSTR lpString2,
#   int    iMaxLength
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcpynA(ql, address, params):
    # Copy String2 into String for max iMaxLength chars
    src = params["lpString2"]
    dst = params["lpString1"]
    max_length = params["iMaxLength"]
    if len(src) > max_length:
        src = src[:max_length]
    ql.mem.write(dst, src.encode())
    return dst


# LPSTR lstrcpynW(
#   LPWSTR  lpString1,
#   LPCWSTR lpString2,
#   int    iMaxLength
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcpynW(ql, address, params):
    # Copy String2 into String for max iMaxLength chars
    src = params["lpString2"]
    dst = params["lpString1"]
    max_length = params["iMaxLength"]
    if len(src) > max_length:
        src = src[:max_length]
    ql.mem.write(dst, src.encode("utf-16le"))
    return dst


# LPSTR lstrcpyA(
#   LPSTR  lpString1,
#   LPCSTR lpString2,
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcpyA(ql, address, params):
    # Copy String2 into String
    src = params["lpString2"]
    dst = params["lpString1"]
    ql.mem.write(dst, src.encode())
    return dst


# LPSTR lstrcpyW(
#   LPSTR  lpString1,
#   LPCSTR lpString2,
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcpyW(ql, address, params):
    # Copy String2 into String
    src = params["lpString2"]
    dst = params["lpString1"]
    ql.mem.write(dst, src.encode("utf-16le"))
    return dst


# LPSTR lstrcatA(
#   LPSTR  lpString1,
#   LPCSTR lpString2
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcatA(ql, address, params):
    # Copy String2 into String
    src = params["lpString2"]
    pointer = params["lpString1"]
    string_base = ql.os.read_cstring(pointer)
    params["lpString1"] = string_base
    result = string_base + src + "\x00"
    ql.mem.write(pointer, result.encode())
    return pointer


# LPSTR lstrcatW(
#   LPWSTR  lpString1,
#   LPCWSTR lpString2
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcatW(ql, address, params):
    # Copy String2 into String
    src = params["lpString2"]
    pointer = params["lpString1"]
    string_base = ql.os.read_wstring(pointer)
    params["lpString1"] = string_base
    result = string_base + src + "\x00"
    ql.mem.write(pointer, result.encode("utf-16le"))
    return pointer


# int lstrcmpiW(
#   LPCWSTR lpString1,
#   LPCWSTR lpString2
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcmpiW(ql, address, params):
    # Copy String2 into String
    str1 = params["lpString1"]
    str2 = params["lpString2"]
    if str1 == str2:
        return 0
    elif str1 > str2:
        return 1
    else:
        return -1


# int lstrcmpiA(
#   LPCSTR lpString1,
#   LPCSTR lpString2
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_lstrcmpiA(ql, address, params):
    return hook_lstrcmpiW.__wrapped__(ql, address, params)


# HRSRC FindResourceA(
#   HMODULE hModule,
#   LPCSTR  lpName,
#   LPCSTR  lpType
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_FindResourceA(ql, address, params):
    # Retrieve a resource
    # Name e Type can be int or strings, this can be a problem
    name = params["lpName"]
    type = params["lpType"]
    # TODO i don't know how to implement this, the return 0 is to simulate an error
    return 0


# BOOL IsBadReadPtr(
#   const VOID *lp,
#   UINT_PTR   ucb
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_IsBadReadPtr(ql, address, params):
    # Check read permission for size of memory
    ACCESS_TRUE = 0
    ACCESS_FALSE = 1
    return ACCESS_TRUE


# BOOL IsBadWritePtr(
#   const VOID *lp,
#   UINT_PTR   ucb
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_IsBadWritePtr(ql, address, params):
    # Check read permission for size of memory
    ACCESS_TRUE = 0
    ACCESS_FALSE = 1
    return ACCESS_TRUE


def compare(p1, operator, p2):
    if operator == "==":
        return p1 == p2
    elif operator == ">":
        return p1 > p2
    elif operator == ">=":
        return p1 >= p2
    elif operator == "<":
        return p1 < p2
    elif operator == "<=":
        return p1 <= p2
    else:
        raise QlErrorNotImplemented("API not implemented")


# typedef struct _OSVERSIONINFOEXA {
#   DWORD dwOSVersionInfoSize;
#   DWORD dwMajorVersion;
#   DWORD dwMinorVersion;
#   DWORD dwBuildNumber;
#   DWORD dwPlatformId;
#   CHAR  szCSDVersion[128];
#   WORD  wServicePackMajor;
#   WORD  wServicePackMinor;
#   WORD  wSuiteMask;
#   BYTE  wProductType;
#   BYTE  wReserved;
# } OSVERSIONINFOEXA, *POSVERSIONINFOEXA, *LPOSVERSIONINFOEXA;


# BOOL VerifyVersionInfoW(
#   LPOSVERSIONINFOEXW lpVersionInformation,
#   DWORD              dwTypeMask,
#   DWORDLONG          dwlConditionMask
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_VerifyVersionInfoW(ql, address, params):
    #  https://docs.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-verifyversioninfow2
    pointer = params["lpVersionInformation"]
    os_asked = OsVersionInfoExA(ql)
    os_asked.read(pointer)
    ConditionMask: dict = ql.os.hooks_variables["ConditionMask"]
    res = True
    for key, value in ConditionMask.items():
        if value == VER_EQUAL:
            operator = "=="
        elif value == VER_GREATER:
            operator = ">"
        elif value == VER_GREATER_EQUAL:
            operator = ">="
        elif value == VER_LESS:
            operator = "<"
        elif value == VER_LESS_EQUAL:
            operator = "<="
        else:
            raise QlErrorNotImplemented("API not implemented with operator %d" % value)
        # Versions should be compared together
        if key == VER_MAJORVERSION or key == VER_MINORVERSION or key == VER_PRODUCT_TYPE:
            major_version_asked = os_asked.major[0]
            minor_version_asked = os_asked.minor[0]
            product_type = os_asked.product[0]
            concat = str(major_version_asked) + str(minor_version_asked) + str(product_type)

            # Just a print for analysts, will remove it from here in the future
            if key == VER_MAJORVERSION:
                ql.log.debug("The Target is checking the windows Version!")
                version_asked = SYSTEMS_VERSION.get(concat, None)
                if version_asked is None:
                    raise QlErrorNotImplemented("API not implemented for version %s" % concat)
                else:
                    ql.log.debug("The target asks for version %s %s" % (operator, version_asked))
            # We can finally compare
            qiling_os = str(ql.os.profile.get("SYSTEM", "majorVersion")) + str(
                ql.os.profile.get("SYSTEM", "minorVersion")) + str(
                ql.os.profile.get("SYSTEM", "productType"))
            res = compare(int(qiling_os), operator, int(concat))
        elif key == VER_SERVICEPACKMAJOR:
            res = compare(ql.os.profile.getint("SYSTEM", "VER_SERVICEPACKMAJOR"), operator, os_asked.service_major[0])
        else:
            raise QlErrorNotImplemented("API not implemented for key %s" % key)
        # The result is a AND between every value, so if we find a False we just exit from the loop
        if not res:
            ql.os.last_error = ERROR_OLD_WIN_VERSION
            return 0
    # reset mask
    ql.os.hooks_variables["ConditionMask"] = {}
    return res


# BOOL GetUserNameW(
#   LPWSTR  lpBuffer,
#   LPDWORD pcbBuffer
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={"lpBuffer": POINTER, "pcbBuffer": POINTER})
def hook_GetUserNameW(ql, address, params):
    username = (ql.os.profile["USER"]["username"] + "\x00").encode("utf-16le")
    dst = params["lpBuffer"]
    max_size = params["pcbBuffer"]
    ql.mem.write(max_size, len(username).to_bytes(4, byteorder="little"))
    if len(username) > max_size:
        ql.os.last_error = ERROR_INSUFFICIENT_BUFFER
        return 0
    else:
        ql.mem.write(dst, username)
    return 1


# BOOL GetUserNameA(
#   LPCSTR  lpBuffer,
#   LPDWORD pcbBuffer
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetUserNameA(ql, address, params):
    username = (ql.os.profile["USER"]["username"] + "\x00").encode()
    dst = params["lpBuffer"]
    max_size = params["pcbBuffer"]
    ql.mem.write(max_size, len(username).to_bytes(4, byteorder="little"))
    if len(username) > max_size:
        ql.os.last_error = ERROR_INSUFFICIENT_BUFFER
        return 0
    else:
        ql.mem.write(dst, username)
    return 1

# BOOL GetUserNameA(
#   LPCSTR  lpBuffer,
#   LPDWORD pcbBuffer
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpBuffer": POINTER,
    "pcbBuffer": POINTER
})
def hook_GetUserNameA(ql, address, params):
    username = (ql.os.profile["USER"]["username"] + "\x00").encode()
    dst = params["lpBuffer"]
    max_size = params["pcbBuffer"]
    ql.mem.write(max_size, len(username).to_bytes(4, byteorder="little"))
    if len(username) > max_size:
        ql.os.last_error = ERROR_INSUFFICIENT_BUFFER
        return 0
    else:
        ql.mem.write(dst, username)
    return 1


# BOOL GetComputerNameW(
#   LPWSTR  lpBuffer,
#   LPDWORD nSize
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetComputerNameW(ql, address, params):
    computer = (ql.os.profile["SYSTEM"]["computername"] + "\x00").encode("utf-16le")
    dst = params["lpBuffer"]
    pointer_to_max_size = params["nSize"]
    nSize = int.from_bytes(ql.mem.read(pointer_to_max_size, ql.pointersize), byteorder="little")
    ql.mem.write(pointer_to_max_size, (len(computer)-2).to_bytes(4, byteorder="little"))
    if len(computer) > nSize:
        #ql.os.last_error = ERROR_BUFFER_OVERFLOW
        return 0
    else:
        ql.mem.write(dst, computer)
    return 1


# BOOL GetComputerNameA(
#   LPCSTR  lpBuffer,
#   LPDWORD nSize
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetComputerNameA(ql, address, params):
    computer = (ql.os.profile["SYSTEM"]["computername"] + "\x00").encode()
    dst = params["lpBuffer"]
    max_size = params["nSize"]
    ql.mem.write(max_size, (len(computer) - 2).to_bytes(4, byteorder="little"))
    if len(computer) > max_size:
        ql.os.last_error = ERROR_BUFFER_OVERFLOW
        return 0
    else:
        ql.mem.write(dst, computer)
    return 1

# DWORD GetPrivateProfileStringA(
#   LPCSTR lpAppName,
#   LPCSTR lpKeyName,
#   LPCSTR lpDefault,
#   LPSTR  lpReturnedString,
#   DWORD  nSize,
#   LPCSTR lpFileName
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpAppName": STRING,
    "lpKeyName": STRING,
    "lpDefault": STRING,
    "lpReturnedString": POINTER,
    "nSize": DWORD,
    "lpFileName": STRING

})
def hook_GetPrivateProfileStringA(ql, address, params):
    lpAppName = params["lpAppName"]
    lpKeyName = params["lpKeyName"]
    lpDefault = params["lpDefault"]
    lpReturnedString = params["lpReturnedString"]
    nSize = params["nSize"]
    lpFileName = params['lpFileName']

    try:
        f = open(lpFileName)
    except:
        ql.os.last_error = ERROR_OLD_WIN_VERSION
        return 0
    config = configparser.ConfigParser()
    config.read_file(f)
    if lpAppName in config and lpKeyName in config[lpAppName]:
        value = (config[lpAppName][lpKeyName]).encode("utf-8")
    else:
        value = lpDefault
    write_len = len(value)
    if write_len > nSize-1:
        write_len = nSize-1
    ql.mem.write(lpReturnedString, value[write_len:] + b"\x00")
    f.close()
    return write_len
    
# BOOL WritePrivateProfileStringA(
#   LPCSTR lpAppName,
#   LPCSTR lpKeyName,
#   LPCSTR lpString,
#   LPCSTR lpFileName
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpAppName": STRING,
    "lpKeyName": STRING,
    "lpString": STRING,
    "lpFileName": STRING

})
def hook_WritePrivateProfileStringA(ql, address, params):
    pass

# UINT GetSystemDirectoryA(
#   LPSTR lpBuffer,
#   UINT  uSize
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpBuffer": POINTER,
    "uSize": DWORD
})
def hook_GetSystemDirectoryA(ql, address, params):
    lpBuffer = params["lpBuffer"]
    uSize = params["uSize"]
    value = "C:\\Windows\\System32\x00".encode("utf-8")
    ql.mem.write(lpBuffer, value)
    return len(value)

# UINT GetWindowsDirectoryA(
#   LPSTR lpBuffer,
#   UINT  uSize
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpBuffer": POINTER,
    "uSize": DWORD
})
def hook_GetWindowsDirectoryA(ql, address, params):
    lpBuffer = params["lpBuffer"]
    uSize = params["uSize"]
    value = "C:\\Windows\x00".encode("utf-8")
    ql.mem.write(lpBuffer, value)
    return len(value)

# DWORD CharLowerBuffA(
#   LPSTR lpsz,
#   DWORD cchLength
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpsz": POINTER,
    "cchLength": DWORD
})
def hook_CharLowerBuffA(ql, address, params):
    lpBuffer = params["lpBuffer"]
    cchLength = params["cchLength"]
    data = ql.mem.read(lpBuffer, cchLength)
    data = data.decode("utf-8")
    data = data.tolower()
    data = data.encode("utf-8")
    ql.mem.write(lpBuffer, data)
    return len(data)

# LPSTR CharLowerA(
#   LPSTR lpsz
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpsz": POINTER,
})
def hook_CharLowerA(ql, address, params):
    lpsz = params["lpsz"]
    if (lpsz >> 16) > 0:
        value = ql.os.read_cstring(lpsz)
        value = value.lower()
        value = value.encode("utf-8")
        ql.mem.write(lpsz, value)
        return len(value)
    else:
        value = chr(lpsz & 0xffff)
        return value.lower()

# void ExitThread(
#   DWORD dwExitCode
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "dwExitCode": DWORD
})
def hook_ExitThread(ql, address, params):
    #ql.emu_stop()
    pass

# BOOL FreeLibrary(
#   HMODULE hLibModule
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "hLibModule": HANDLE
})
def hook_FreeLibrary(ql, address, params):
    return 1

# BOOL DeleteFileA(
#   LPCSTR lpFileName
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "lpFileName": STRING
})
def hook_DeleteFileA(ql, address, params):
    return 1




#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *


dllname = 'kernel32_dll'

# void InitializeSListHead(
#   PSLIST_HEADER ListHead
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_InitializeSListHead(ql, address, params):
    addr = params["ListHead"]
    handle = Handle(obj=[], id=addr)
    ql.os.handle_manager.append(handle)
    return 0



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#
from .advapi32 import *
from .msvcrt import *
from .user32 import *
from .wininet import *
from .msi import *
from .ntdll import *
from .ucrtbased import *
from .wsock32 import *
from .ole32 import *
from .shlwapi import *
from .crypt32 import *
from .mscoree import *
from .msvbvm60 import *
from .shell32 import *
from .ntoskrnl import *
from .wudplatform import *
from .oleaut32 import *
from .kernel32 import *



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

STDCALL = 1
CDECL = 2

DWORD = 1
UINT = 1
INT = 1
BOOL = 1
SIZE_T = 1
BYTE = 1
ULONGLONG = 2
HANDLE = 3
POINTER = 3
STRING = 4
WSTRING = 5
GUID = 6

# OS Threading Constants
THREAD_EVENT_INIT_VAL = 0
THREAD_EVENT_EXIT_EVENT = 1
THREAD_EVENT_UNEXECPT_EVENT = 2
THREAD_EVENT_EXECVE_EVENT = 3
THREAD_EVENT_CREATE_THREAD = 4
THREAD_EVENT_BLOCKING_EVENT = 5
THREAD_EVENT_EXIT_GROUP_EVENT = 6

THREAD_STATUS_RUNNING = 0
THREAD_STATUS_BLOCKING = 1
THREAD_STATUS_TERMINATED = 2
THREAD_STATUS_STOPPED = 4
THREAD_STATUS_TIMEOUT = 3
THREAD_STATUS_SUSPEND = 5

reptypedict = {
            "BSTR": "POINTER",
            "DLGPROC": "POINTER",
            "DWORDLONG": "ULONGLONG",
            "DWORD_PTR": "POINTER",
            "GROUP": "INT",
            "HBITMAP": "HANDLE",
            "HDC": "POINTER",
            "HEAP_INFORMATION_CLASS": "UINT",
            "HGLOBAL": "POINTER",
            "HIMAGELIST": "HANDLE",
            "HHOOK": "POINTER",
            "HINSTANCE": "HANDLE",
            "HINTERNET": "POINTER",
            "HKEY": "HANDLE",
            "HLOCAL": "POINTER",
            "HMODULE": "HANDLE",
            "HOOKPROC": "POINTER",
            "HRSRC": "POINTER",
            "HWND": "HANDLE",
            "INSTALLSTATE": "POINTER",
            "INTERNET_PORT": "DWORD",
            "INT_PTR": "POINTER",
            "LARGE_INTEGER": "POINTER",
            "LCID": "POINTER",
            "LONG": "ULONGLONG",
            "LPARAM": "POINTER",
            "LPBOOL": "POINTER",
            "LPBYTE": "POINTER",
            "LPCCH": "POINTER",
            "LPCONTEXT": "POINTER",
            "LPCPINFO": "POINTER",
            "LPCRITICAL_SECTION": "POINTER",
            "LPCSTR": "STRING",
            "LPCVOID": "POINTER",
            "LPCWCH": "POINTER",
            "LPCWSTR": "WSTRING",
            "LPDWORD": "POINTER",
            "LPFILETIME": "POINTER",
            "LPINTERNET_BUFFERSA": "POINT",
            "LPMESSAGEFILTER": "POINTER",
            "LPMODULEINFO": "POINTER",
            "LPNLSVERSIONINFO": "POINTER",
            "LPOSVERSIONINFOA": "STRING",
            "LPOSVERSIONINFOEXW": "POINTER",
            "LPOSVERSIONINFOW": "WSTRING",
            "LPOVERLAPPED": "POINTER",
            "LPPOINT": "POINTER",
            "LPPROCESSENTRY32W": "POINTER",
            "LPSECURITY_ATTRIBUTES": "POINTER",
            "LPSTARTUPINFOA": "POINTER",
            "LPSTARTUPINFOW": "POINTER",
            "LPSTR": "POINTER",
            "LPSTREAM": "POINTER",
            "LPSYSTEMTIME": "POINTER",
            "LPSYSTEM_INFO": "POINTER",
            "LPTHREAD_START_ROUTINE": "POINTER",
            "LPTOP_LEVEL_EXCEPTION_FILTER": "DWORD",
            "LPUNKNOWN": "POINTER",
            "LPVOID": "POINTER",
            "LPWCH": "POINTER",
            "LPWIN32_FIND_DATAA": "POINTER",
            "LPWORD": "POINTER",
            "LPWSADATA": "STRING",
            "LPWSAPROTOCOL_INFOA": "POINTER",
            "LPWSTR": "POINTER",
            "MSIHANDLE": "POINTER",
            "OBJECT_INFORMATION_CLASS": "INT",
            "OLECHAR": "WSTRING",
            "PBOOL": "POINTER",
            "PCACTCTXW": "POINTER",
            "PCNZCH": "STRING",
            "PDWORD": "POINTER",
            "PFLS_CALLBACK_FUNCTION": "POINTER",
            "PHKEY": "POINTER",
            "PMEMORY_BASIC_INFORMATION": "POINTER",
            "PROCESSINFOCLASS": "INT",
            "PSECURITY_DESCRIPTOR": "POINTER",
            "PSID": "HANDLE",
            "PSID_IDENTIFIER_AUTHORITY": "POINTER",
            "PSLIST_HEADER": "POINTER",
            "PSRWLOCK": "POINTER",
            "PULONG": "POINTER",
            "PVECTORED_EXCEPTION_HANDLER": "HANDLE",
            "PVOID": "POINTER",
            "REFCLSID": "POINTER",
            "REFIID": "POINTER",
            "REGSAM": "POINTER",
            "SC_HANDLE": "HANDLE",
            "SHELLEXECUTEINFOA": "POINTER",
            "SHELLEXECUTEINFOW": "POINTER",
            "SHFILEINFOW": "POINTER",
            "SOCKET": "INT",
            "SOLE_AUTHENTICATION_SERVICE": "POINTER",
            "TOKEN_INFORMATION_CLASS": "DWORD",
            "UINT_PTR": "POINTER",
            "ULONG": "UINT",
            "ULONG_PTR": "POINTER",
            "WORD": "DWORD",
            "WPARAM": "UINT",
            "_EXCEPTION_POINTERS": "POINTER",
            "int": "INT",
            "size_t": "UINT",
            "sockaddr": "POINTER",
            "unsigned int": "UINT",
            "void": "POINTER"
}
